                                                                        CA100010
*ASM XOPTS(EXCI)                                                        CA100020
*********************************************************************** CA100030
*   PROGRAM: CA1DCALL - INTERFACE BETWEEN REXX EXEC AND CICS          * CA100040
*   LANGUAGE: ASSEMBLER                                               * CA100050
*                                                                     * CA100060
*   FUNCTION: PROVIDE CICS EXCI DYNAMIC PROGRAM LINK                  * CA100070
*                                                                     * CA100080
*   FORMAT OF CALL FROM REXX:                                         * CA100090
*     CA1DCALL(PROGRAM(PROGRAM_NAME) ,                                * CA100100
*              APPLID(VTAM_APPLID) ,                                  * CA100110
*              USERID(USER_ID) ,                                      * CA100120
*              COMMAREA(CA_NAME) ,          OPTIONAL                  * CA100130
*              LENGTH(CA_LENGTH) ,          OPTIONAL                  * CA100140
*              DATALENGTH(CA_DATALENGTH) ,  OPTIONAL                  * CA100150
*              TRANSID(TRANS_ID)            OPTIONAL                  * CA100160
*                                                                     * CA100170
*   WHERE:                                                            * CA100180
*     PROGRAM_NAME IS THE CICS PROGRAM NAME TO BE CALLED              * CA100190
*     VTAM_APPLID IS THE VTAM APPLID FOR THE CICS REGION TO ROUTE     * CA100200
*       THE REQUEST TO.                                               * CA100210
*     USER_ID IS THE USER'S ID WHICH HAS BEEN SECURITY CHECKED.       * CA100220
*     CA_NAME IS THE REXX VARIABLE NAME WHICH IS THE COMMAREA DATA    * CA100230
*       IF NOT SPECIFIED, THEN ASSUME NO COMMAREA TO BE RETURNED.     * CA100240
*     CA_LENGTH IS THE LENGTH OF THE COMMAREA                         * CA100250
*       IF NOT SPECIFIED THEN ASSUME LENGTH OF REXX COMMAREA VARIABLE.* CA100260
*     CA_DATALENGTH IS THE LENGTH OF THE COMMAREA TO BE PASSED        * CA100270
*       STARTING WITH THE FIRST CHARACTER OF THE COMMAREA.            * CA100280
*     TRANS_ID IS THE TRANSACTION NAME FOR THE MIRROR TRANSACTION.    * CA100290
*       CSMI USED IF NOT ENTERED                                      * CA100300
*                                                                     * CA100310
*   REXX VARIABLES USED:                                              * CA100320
*     CA_NAME WILL BE REPLACED UPON SUCCESSFUL EXECUTION OF THE       * CA100330
*       CALLED PROGRAM WITH THE RETURNED COMMAREA.                    * CA100340
*     EXCI_RESP IS THE RESP CODE RETURNED BY THE EXCI.  FORMAT IS     * CA100350
*       DECIMAL NUMERIC.                                              * CA100360
*     EXCI_RESP2 IS THE RESP2 CODE RETURNED BY THE EXCI.  FORMAT IS   * CA100370
*       DECIMAL NUMERIC.                                              * CA100380
*     EXCI_ABCODE IS THE ABCODE RETURNED BY THE EXCI.  FORMAT IS      * CA100390
*       4 BYTES CHARACTER.                                            * CA100400
*     EXCI_MESSAGE IS ANY MESSAGE RETURNED BY THE EXCI.  THIS AREA    * CA100410
*       WILL CONTAIN A NULL STRING IF CICS DOES NOT RETURN A MESSAGE. * CA100420
*                                                                     * CA100430
*   RETURN CODES: (RESULT)                                            * CA100440
*     -1 = PROGRAM NAME NOT SPECIFIED OR INVALID                      * CA100450
*     -2 = VTAM APPLID NOT SPECIFIED OR INVALID                       * CA100460
*     -3 = COMMAREA NOT AVAILABLE                                     * CA100470
*     -4 = LOAD OF IRXEXCOM FAILED                                    * CA100480
*     -5 = INVALID PARAMETER TYPE ENTERED                             * CA100490
*     -6 = TRANSID INVALID                                            * CA100500
*     -7 = LENGTH INVALID                                             * CA100510
*     -8 = DATALENGTH INVALID                                         * CA100520
*     -9 = INVALID REXX NAME FOR COMMAREA SPECIFIED                   * CA100530
*    -10 = INVALID USERID SPECIFIED                                   * CA100540
*      0 = PROGRAM RECEIVED NORMAL RETURN FROM EXEC CICS EXCI         * CA100550
*          EXCI_... VARIABLES CONTAIN RESULTING RETURN CODES          * CA100560
*          (SEE EXTERNAL CICS INTERFACE FOR RETURNED VARIABLES.)      * CA100570
*                                                                     * CA100580
*   REGISTER USAGE:                                                   * CA100590
*      1 - WORK REGISTER                                              * CA100600
*      2 - POINTER TO PARAMETERS PASSED BY REXX                       * CA100610
*      9 - BRANCH AND LINK REGISTER TO SUBROUTINES                    * CA100620
*     12 - BASE REGISTER                                              * CA100630
*     13 - DFHEISTG REGISTER (DYNAMIC STORAGE AREA)                   * CA100640
*     14 - WORK REGISTER                                              * CA100650
*     15 - WORK REGISTER                                              * CA100660
*********************************************************************** CA100670
*********************************************************************** CA100680
*   INCLUDE THE STANDARD COPYBOOKS FOR EXCI DSECTS                    * CA100690
*********************************************************************** CA100700
          COPY DFHXCRCD                                                 CA100710
          EJECT                                                         CA100720
          COPY DFHXCPLD                                                 CA100730
          EJECT                                                         CA100740
*********************************************************************** CA100750
*   INCLUDE THE REXX COPYBOOKS FOR DSECTS                             * CA100760
*********************************************************************** CA100770
          IRXSHVB                 SHARED VARIABLE BLOCK                 CA100780
          EJECT                                                         CA100790
          IRXEVALB                EVALUATION BLOCK                      CA100800
          EJECT                                                         CA100810
          IRXEFPL                 EXTERNAL FUNCTION PARAMETER LIST      CA100820
          EJECT                                                         CA100830
          IRXARGTB                PARSED PARAMETER ARGUMENT LIST        CA100840
          EJECT                                                         CA100850
*********************************************************************** CA100860
*        CA1DCALL PROGRAM START                                       * CA100870
*********************************************************************** CA100880
CA1DCALL DFHEIENT CODEREG=(12),DATAREG=(13)                             CA100890
CA1DCALL AMODE 31                                                       CA100900
CA1DCALL RMODE ANY                                                      CA100910
         LR    2,1                SET R2 TO PARM REG                    CA100920
         USING EFPL,2             SET ADDRESSABILITY                    CA100930
         L     1,DFHEISTG+4       LOAD OLD SAVEAREA ADDRESS             CA100940
         L     1,20(0,1)          LOAD REXX ENVBLOCK ADDRESS            CA100950
         ST    1,ENVBADDR         SAVE REXX ENVBLOCK ADDRESS            CA100960
*********************************************************************** CA100970
*        MAIN CODE FOR PROGRAM                                        * CA100980
*********************************************************************** CA100990
         MVC   T,=C'BEFORE CALL'  SET BEFORE CALL                       CA101000
         MVC   TL,=F'11'          SET LENGTH                            CA101010
         SR    1,1                CLEAR WORK REGISTER                   CA101020
         ST    1,RETCODE          SET ZERO RETURN CODE                  CA101030
         ST    1,BUFADDR          SET ZERO ADDRESS                      CA101040
         BAL   9,LOAD_IRXEXCOM    GO GET IRXEXCOM ROUTINE ADDRESS       CA101050
         BAL   9,PARSEPRM         GO PARSE PARAMETERS                   CA101060
         BAL   9,GETBUFF          GET BUFFER FOR COMMAREA               CA101070
         L     1,CANAMEP          LOAD POINTER TO COMMAREA NAME         CA101080
         LTR   1,1                ANY POINTER THERE?                    CA101090
         BZ    CALL               ..NO, SKIP GETTING COMMAREA           CA101100
         BAL   9,GETCA            GET COMMAREA FROM REXX                CA101110
CALL     DS    0H                                                       CA101120
         BAL   9,CALLCICS         ISSUE EXCI LINK TO CICS               CA101130
         BAL   9,REXXOUT          GO FORMAT AND WRITE REXX VARIABLES    CA101140
*********************************************************************** CA101150
*        RETURN PARSE ERROR TO CALLER                                 * CA101160
*********************************************************************** CA101170
RETURN   DS    0H                                                       CA101180
         L     1,RETCODE          LOAD RESPONSE CODE                    CA101190
         BAL   9,CVT_NUM          CONVERT TO REXX VALUE                 CA101200
         L     3,EFPLEVAL                                               CA101210
         L     3,0(0,3)           LOAD EVALBLOCK ADDRESS                CA101220
         USING EVALBLOCK,3        ESTABLISH ADDRESSABILITY              CA101230
         L     14,EVALBLOCK_EVSIZE                LOAD BLOCK SIZE       CA101240
         SLL   14,3                               MULTIPLY BY 8         CA101250
         S     14,=F'16'                          REDUCE BY 16          CA101260
         C     14,=F'16'                          IS AREA > 16?         CA101270
         BL    RETURN9                      ..NO, DO NOT SET EVALBLOCK  CA101280
         MVC   EVALBLOCK_EVDATA(16),CHARNUM       MOVE IN VALUE         CA101290
         MVC   EVALBLOCK_EVLEN,CHARNUM_LEN        SET LENGTH OF VALUE   CA101300
         DROP  3                                                        CA101310
RETURN9  DS    0H                                                       CA101320
         LA    14,TRACE_LOC       LOAD NAME POINTER                     CA101330
         ST    14,NAMEPTR         SAVE FOR FORMAT                       CA101340
         L     14,TRACE_LOC_L     LOAD NAME LENGTH                      CA101350
         ST    14,NAMELEN         SAVE FOR FORMAT                       CA101360
         LA    14,T               LOAD VALUE PTR                        CA101370
         ST    14,VALUPTR         SAVE FOR FORMAT                       CA101380
         L     14,TL              LOAD VALUE LENGHT                     CA101390
         ST    14,VALULEN         SAVE FOR FORMAT                       CA101400
         SR    14,14              CLEAR 14                              CA101410
         ST    14,SHVBKID         SET SHVBLOCK INDEX TO 0               CA101420
         BAL   9,FORMAT_SHVB      FORMAT THE REXX SHVBLOCK              CA101430
         BAL   9,WRITE_REXX       GO STORE REXX VARIABLES               CA101440
*********************************************************************** CA101450
*        RETURN TO CALLER                                             * CA101460
*********************************************************************** CA101470
         BAL   9,FREEBUFF         FREE AREA FOR COMMAREA                CA101480
         SR    15,15              SET RETURN CODE TO ZERO               CA101490
         DFHEIRET RCREG=(15)                                            CA101500
         EJECT                                                          CA101510
*********************************************************************** CA101520
*   GETMAIN AREA TO BE USED FOR COMMAREA                              * CA101530
*                                                                     * CA101540
*   REGISTER USAGE:                                                   * CA101550
*      0 - LENGTH OF AREA TO GETMAIN                                  * CA101560
*      1 - RETURNED ADDRESS OF AREA                                   * CA101570
*      9 - RETURN TO CALLER ADDRESS                                   * CA101580
*     14 - WORK REGISTER                                              * CA101590
*     15 - WORK REGISTER AND RETURN CODE                              * CA101600
*********************************************************************** CA101610
GETBUFF  DS    0H                                                       CA101620
         L     0,=F'32768'        GET AREA FOR MAX COMMAREA             CA101630
         GETMAIN R,LV=(0)         GET THE AREA                          CA101640
         ST    1,BUFADDR          SAVE BUFFER AREA ADDRESS              CA101650
         BR    9                  RETURN TO CALLER                      CA101660
         SPACE 2                                                        CA101670
*********************************************************************** CA101680
*   FREEMAIN AREA USED FOR COMMAREA                                   * CA101690
*                                                                     * CA101700
*   REGISTER USAGE:                                                   * CA101710
*      0 - LENGTH OF AREA TO FREEMAIN                                 * CA101720
*      1 - ADDRESS OF AREA TO FREEMAIN                                * CA101730
*      9 - RETURN TO CALLER ADDRESS                                   * CA101740
*     14 - WORK REGISTER                                              * CA101750
*     15 - WORK REGISTER AND RETURN CODE                              * CA101760
*********************************************************************** CA101770
FREEBUFF DS    0H                                                       CA101780
         L     0,=F'32768'        GET AREA FOR MAX COMMAREA             CA101790
         L     1,BUFADDR          LOAD ADDRESS OF BUFFER                CA101800
         LTR   1,1                IS THERE AN ADDRESS TO FREE?          CA101810
         BZR   9                  ..NO, RETURN TO CALLER                CA101820
         FREEMAIN R,LV=(0),A=(1)  FREE THE AREA                         CA101830
         BR    9                  RETURN TO CALLER                      CA101840
         EJECT                                                          CA101850
*********************************************************************** CA101860
*   FORMAT SHVBLOCKS AND WRITE REXX VARIABLES                         * CA101870
*                                                                     * CA101880
*   REGISTER USAGE:                                                   * CA101890
*      0 - LENGTH OF AREA TO FREEMAIN                                 * CA101900
*      1 - ADDRESS OF AREA TO FREEMAIN                                * CA101910
*      9 - RETURN TO CALLER ADDRESS AND BRANCH AND LINK REGISTER      * CA101920
*     14 - WORK REGISTER                                              * CA101930
*     15 - WORK REGISTER                                              * CA101940
*********************************************************************** CA101950
REXXOUT  DS    0H                                                       CA101960
*                                                                       CA101970
*        FORMAT BLOCK 0 (EXCI_RESP)                                     CA101980
*                                                                       CA101990
         ST    9,REXXOUT_RET      SAVE RETURN ADDRESS                   CA102000
         LA    14,EXCI_RESP       LOAD NAME POINTER                     CA102010
         ST    14,NAMEPTR         SAVE                                  CA102020
         L     14,EXCI_RESP_L     LOAD NAME LENGTH                      CA102030
         ST    14,NAMELEN         SAVE                                  CA102040
         L     1,RCRESP           LOAD VALUE POINTER                    CA102050
         BAL   9,CVT_NUM          CONVERT TO REXX VALUE                 CA102060
         MVC   RCRESPC,CHARNUM    SAVE CHARACTER VALUE OF RCRESP        CA102070
         LA    14,RCRESPC         POINT TO CHARACTER AREA               CA102080
         ST    14,VALUPTR         SAVE                                  CA102090
         L     14,CHARNUM_LEN     LOAD VALUE LENGTH                     CA102100
         ST    14,VALULEN         SAVE                                  CA102110
         SR    14,14              SET 0 BLOCK INDICATOR                 CA102120
         ST    14,SHVBKID         SAVE                                  CA102130
         BAL   9,FORMAT_SHVB      GO FORMAT BLOCK                       CA102140
*                                                                       CA102150
*        FORMAT BLOCK 1 (EXCI_RESP2)                                    CA102160
*                                                                       CA102170
         LA    14,EXCI_RESP2      LOAD NAME POINTER                     CA102180
         ST    14,NAMEPTR         SAVE                                  CA102190
         L     14,EXCI_RESP2_L    LOAD NAME LENGTH                      CA102200
         ST    14,NAMELEN         SAVE                                  CA102210
         L     1,RCRESP2          LOAD VALUE POINTER                    CA102220
         BAL   9,CVT_NUM          CONVERT TO REXX VALUE                 CA102230
         MVC   RCRESP2C,CHARNUM   SAVE CHARACTER VALUE OF RCRESP2       CA102240
         LA    14,RCRESP2C        POINT TO RETURNED VALUE               CA102250
         ST    14,VALUPTR         SAVE                                  CA102260
         L     14,CHARNUM_LEN     LOAD VALUE LENGTH                     CA102270
         ST    14,VALULEN         SAVE                                  CA102280
         L     14,SHVBKID         LOAD CURRENT BLOCK INDICATOR          CA102290
         LA    14,1(14)           INCREMENT BY 1                        CA102300
         ST    14,SHVBKID         SAVE                                  CA102310
         BAL   9,FORMAT_SHVB      GO FORMAT BLOCK                       CA102320
*                                                                       CA102330
*        FORMAT BLOCK 2 (EXCI_ABCODE)                                   CA102340
*                                                                       CA102350
         LA    14,EXCI_ABCODE     LOAD NAME POINTER                     CA102360
         ST    14,NAMEPTR         SAVE                                  CA102370
         L     14,EXCI_ABCODE_L   LOAD NAME LENGTH                      CA102380
         ST    14,NAMELEN         SAVE                                  CA102390
         LA    14,RCABCODE        LOAD VALUE POINTER                    CA102400
         ST    14,VALUPTR         SAVE                                  CA102410
         L     14,=F'4'           LOAD VALUE LENGTH                     CA102420
         ST    14,VALULEN         SAVE                                  CA102430
         L     14,SHVBKID         LOAD CURRENT BLOCK INDICATOR          CA102440
         LA    14,1(14)           INCREMENT BY 1                        CA102450
         ST    14,SHVBKID         SAVE                                  CA102460
         BAL   9,FORMAT_SHVB      GO FORMAT BLOCK                       CA102470
*                                                                       CA102480
*        FORMAT BLOCK 3 (EXCI_MESSAGE)                                  CA102490
*                                                                       CA102500
         LA    14,EXCI_MESSAGE    LOAD NAME POINTER                     CA102510
         ST    14,NAMEPTR         SAVE                                  CA102520
         L     14,EXCI_MESSAGE_L  LOAD NAME LENGTH                      CA102530
         ST    14,NAMELEN         SAVE                                  CA102540
         LA    14,RCMSGPTR        LOAD VALUE POINTER                    CA102550
         ST    14,VALUPTR         SAVE                                  CA102560
         L     14,RCMSGLEN        LOAD MESSAGE LENGTH                   CA102570
         ST    14,VALULEN         SAVE VALUE LENGTH                     CA102580
         L     14,SHVBKID         LOAD CURRENT BLOCK INDICATOR          CA102590
         LA    14,1(14)           INCREMENT BY 1                        CA102600
         ST    14,SHVBKID         SAVE                                  CA102610
         BAL   9,FORMAT_SHVB      GO FORMAT BLOCK                       CA102620
*                                                                       CA102630
*        FORMAT BLOCK 4 (DPLRESP)                                       CA102640
*                                                                       CA102650
         LA    14,DPL_RESP        LOAD NAME POINTER                     CA102660
         ST    14,NAMEPTR         SAVE                                  CA102670
         L     14,DPL_RESP_L      LOAD NAME LENGTH                      CA102680
         ST    14,NAMELEN         SAVE                                  CA102690
         L     1,DPLRESP          LOAD VALUE                            CA102700
         BAL   9,CVT_NUM          CONVERT TO REXX VALUE                 CA102710
         MVC   DPLRESPC,CHARNUM   SAVE CHARACTER VALUE OF DPLRESP       CA102720
         LA    14,DPLRESPC        POINT TO RETURNED VALUE               CA102730
         ST    14,VALUPTR         SAVE                                  CA102740
         L     14,CHARNUM_LEN     LOAD VALUE LENGTH                     CA102750
         ST    14,VALULEN         SAVE                                  CA102760
         L     14,SHVBKID         LOAD CURRENT BLOCK INDICATOR          CA102770
         LA    14,1(14)           INCREMENT BY 1                        CA102780
         ST    14,SHVBKID         SAVE                                  CA102790
         BAL   9,FORMAT_SHVB      GO FORMAT BLOCK                       CA102800
*                                                                       CA102810
*        FORMAT BLOCK 5 (DPLRESP2)                                      CA102820
*                                                                       CA102830
         LA    14,DPL_RESP2       LOAD NAME POINTER                     CA102840
         ST    14,NAMEPTR         SAVE                                  CA102850
         L     14,DPL_RESP2_L     LOAD NAME LENGTH                      CA102860
         ST    14,NAMELEN         SAVE                                  CA102870
         L     1,DPLRESP2         LOAD VALUE                            CA102880
         BAL   9,CVT_NUM          CONVERT TO REXX VALUE                 CA102890
         MVC   DPLRESP2C,CHARNUM  SAVE CHARACTER VALUE OF DPLRESP2      CA102900
         LA    14,DPLRESP2C       POINT TO RETURNED VALUE               CA102910
         ST    14,VALUPTR         SAVE                                  CA102920
         L     14,CHARNUM_LEN     LOAD VALUE LENGTH                     CA102930
         ST    14,VALULEN         SAVE                                  CA102940
         L     14,SHVBKID         LOAD CURRENT BLOCK INDICATOR          CA102950
         LA    14,1(14)           INCREMENT BY 1                        CA102960
         ST    14,SHVBKID         SAVE                                  CA102970
         BAL   9,FORMAT_SHVB      GO FORMAT BLOCK                       CA102980
*                                                                       CA102990
*        FORMAT BLOCK 6 (DPLABEND)                                      CA103000
*                                                                       CA103010
         LA    14,DPL_ABCODE      LOAD NAME POINTER                     CA103020
         ST    14,NAMEPTR         SAVE                                  CA103030
         L     14,DPL_ABCODE_L    LOAD NAME LENGTH                      CA103040
         ST    14,NAMELEN         SAVE                                  CA103050
         LA    14,DPLABEND        LOAD VALUE POINTER                    CA103060
         ST    14,VALUPTR         SAVE                                  CA103070
         L     14,=F'4'           LOAD VALUE LENGTH                     CA103080
         ST    14,VALULEN         SAVE                                  CA103090
         L     14,SHVBKID         LOAD CURRENT BLOCK INDICATOR          CA103100
         LA    14,1(14)           INCREMENT BY 1                        CA103110
         ST    14,SHVBKID         SAVE                                  CA103120
         BAL   9,FORMAT_SHVB      GO FORMAT BLOCK                       CA103130
*                                                                       CA103140
*        FORMAT BLOCK 7 (COMMAREA)                                      CA103150
*                                                                       CA103160
         L     15,CA_LENGTH       LOAD COMMAREA LENGTH                  CA103170
         LTR   15,15              ANY AREA SPECIFIED?                   CA103180
         BZ    REXXOUT9           ..NO, FINISHED FORMATTING             CA103190
         L     14,CANAMEP         LOAD PTR TO NAME                      CA103200
         ST    14,NAMEPTR         SAVE                                  CA103210
         L     14,CANAMEL         LOAD LENGTH OF NAME                   CA103220
         ST    14,NAMELEN         SAVE                                  CA103230
         L     14,BUFADDR         LOAD PTR TO COMMAREA                  CA103240
         ST    14,VALUPTR         SAVE                                  CA103250
         L     14,CA_LENGTH       LOAD VALUE LENGTH                     CA103260
         ST    14,VALULEN         SAVE                                  CA103270
         L     14,SHVBKID         LOAD CURRENT BLOCK INDICATOR          CA103280
         LA    14,1(14)           INCREMENT BY 1                        CA103290
         ST    14,SHVBKID         SAVE                                  CA103300
         ST    14,SHVBKID         SAVE                                  CA103310
         BAL   9,FORMAT_SHVB      GO FORMAT BLOCK                       CA103320
REXXOUT9 DS    0H                                                       CA103330
         BAL   9,WRITE_REXX       GO WRITE REXX VARIABLES               CA103340
         L     9,REXXOUT_RET      LOAD RETURN ADDRESS                   CA103350
         BR    9                  RETURN TO CALLER                      CA103360
         EJECT                                                          CA103370
*********************************************************************** CA103380
*   PARSE PARAMETERS PASSED FROM REXX                                 * CA103390
*                                                                     * CA103400
*   FORMAT FOR CALL FROM REXX EXEC IS:                                * CA103410
*                                                                     * CA103420
*   X = CA1DCALL( PROGRAM(PGMNAME),                                   * CA103430
*                 APPLID(APPL_ID),                                    * CA103440
*                 COMMAREA(CA_VARNAME),           OPTIONAL            * CA103450
*                 TRANSID(TRAN_ID),               OPTIONAL            * CA103460
*                 DATALENGTH(DATA_LENGTH_VALUE),  OPTIONAL            * CA103470
*               )                                                     * CA103480
*                                                                     * CA103490
*   WHERE:                                                            * CA103500
*                                                                     * CA103510
*     PGMNAME IS THE NAME OF THE CICS PROGRAM TO LINK TO              * CA103520
*     CA_VARNAME IS THE NAME OF THE REXX VARIABLE WHICH IS THE        * CA103530
*       THE COMMAREA                                                  * CA103540
*     TRAN_ID IS THE TRANSACTION ID FOR THE MIRROR                    * CA103550
*     APPL_ID IS THE VTAM APPLID OF THE CICS REGION TO ROUTE TO       * CA103560
*     DATA_LENGTH_VALUE IS THE DATALENGTH TO PASS FOR THE COMMAREA    * CA103570
*                                                                     * CA103580
*   REGISTER USAGE:                                                   * CA103590
*      0 - WORK REGISTER                                              * CA103600
*      1 - WORK REGISTER                                              * CA103610
*      3 - ARGUMENT TABLE POINTER                                     * CA103620
*      4 - ARGUMENT POINTER                                           * CA103630
*      5 - ARGUMENT LENGTH                                            * CA103640
*      6 - WORK REGISTER                                              * CA103650
*      7 - WORK REGISTER                                              * CA103660
*      9 - RETURN REGISTER                                            * CA103670
*     15 - WORK REGISTER                                              * CA103680
*********************************************************************** CA103690
PARSEPRM DS    0H                                                       CA103700
         L     3,EFPLARG          POINT TO ARGS TABLE                   CA103710
         USING ARGTABLE_ENTRY,3   ESTABLISH ADDRESSABILITY              CA103720
*        INITIALIZE FIELDS                                              CA103730
         MVC   PGMNAME,=CL8' '    CLEAR AREA WITH BLANKS                CA103740
         MVC   APPLID,=CL8' '     CLEAR AREA WITH BLANKS                CA103750
         MVC   USERID,=CL8' '     CLEAR AREA WITH BLANKS                CA103760
         MVC   TRANSID,=CL4' '    CLEAR AREA WITH BLANKS                CA103770
         SR    15,15              CLEAR REG 15                          CA103780
         ST    15,RETCODE         SET GOOD RETURN CODE                  CA103790
         ST    15,CANAMEP         SAVE IN CA NAME PTR                   CA103800
         ST    15,CANAMEL         SAVE IN CA NAME LENGTH                CA103810
         ST    15,CA_LENGTH       SAVE IN LENGTH VALUE                  CA103820
         ST    15,CA_DATALENGTH   SAVE IN DATALENGTH VALUE              CA103830
         ST    15,CA_RETVD_LEN    SAVE RETRIEVED COMMAREA LENGTH        CA103840
PARS001  DS    0H                 START OF LOOP                         CA103850
         CLC   ARGTABLE_ARGSTRING_PTR(8),=XL8'FFFFFFFFFFFFFFFF'         CA103860
         BE    PARS080            ALL PARAMETERS PROCESSED              CA103870
         L     4,ARGTABLE_ARGSTRING_PTR     LOAD POINTER TO ARGUMENT    CA103880
         L     5,ARGTABLE_ARGSTRING_LENGTH  LOAD ARGUMENT LENGTH        CA103890
         LTR   5,5                TEST FOR ZERO LENGTH                  CA103900
         BZ    PARS070            ZERO LENGTH, GO TO NEXT ARGUMENT      CA103910
         LR    6,4                SAVE STARTING ADDRESS                 CA103920
         SR    7,7                CLEAR PARM PIECE LENGTH               CA103930
PARS002  DS    0H                 START OF LOOP                         CA103940
         CLI   0(4),C' '          LOOK FOR LEADING BLANKS               CA103950
         BNE   PARS003            NOT A BLANK, CONTINUE                 CA103960
         LA    4,1(0,4)           BLANK SKIP IT                         CA103970
         LR    6,4                SAVE NEW STARTING POSITION            CA103980
         BCT   5,PARS002          GO LOOK AT NEXT CHARACTER             CA103990
         B     PARS070            NO MORE CHARACTERS LEFT               CA104000
PARS003  DS    0H                                                       CA104010
         CLI   0(4),C'('          LOOK FOR OPEN PAREN                   CA104020
         BE    PARS010            FOUND OPEN PAREN                      CA104030
         LA    4,1(0,4)           POINT TO NEXT CHARACTER               CA104040
         LA    7,1(0,7)           INCREASE COUNT OF CHARS               CA104050
         BCT   5,PARS003          LOOK AT NEXT CHARACTER                CA104060
         B     PARS070            NO '(' FOUND, GO TO NEXT ARGUMENT     CA104070
PARS010  DS    0H                                                       CA104080
         CLC   =CL7'PROGRAM',0(6) PROGRAM PARAMETER?                    CA104090
         BNE   PARS011            NO, CHECK FOR NEXT TYPE               CA104100
         MVI   TYPEFLG,C'P'       SET FOR PROGRAM NAME                  CA104110
         B     PARS020            GO CHECK FOR VALUE                    CA104120
PARS011  DS    0H                                                       CA104130
         CLC   =CL6'APPLID',0(6)  APPLID PARAMETER?                     CA104140
         BNE   PARS012            NO, CHECK FOR NEXT TYPE               CA104150
         MVI   TYPEFLG,C'A'       SET FOR APPLID NAME                   CA104160
         B     PARS020            GO CHECK FOR VALUE                    CA104170
PARS012  DS    0H                                                       CA104180
         CLC   =CL6'USERID',0(6)  USERID PARAMETER?                     CA104190
         BNE   PARS013            NO, CHECK FOR NEXT TYPE               CA104200
         MVI   TYPEFLG,C'U'       SET FOR USERID NAME                   CA104210
         B     PARS020            GO CHECK FOR VALUE                    CA104220
PARS013  DS    0H                                                       CA104230
         CLC   =CL8'COMMAREA',0(6) COMMAREA PARAMETER?                  CA104240
         BNE   PARS014            NO, CHECK FOR NEXT TYPE               CA104250
         MVI   TYPEFLG,C'C'       SET FOR COMMAREA NAME                 CA104260
         B     PARS020            GO CHECK FOR VALUE                    CA104270
PARS014  DS    0H                                                       CA104280
         CLC   =CL10'DATALENGTH',0(6)  DATALENGTH PARAMETER?            CA104290
         BNE   PARS015            NO, CHECK FOR NEXT TYPE               CA104300
         MVI   TYPEFLG,C'D'       SET FOR DATALENGTH NAME               CA104310
         B     PARS020            GO CHECK FOR VALUE                    CA104320
PARS015  DS    0H                                                       CA104330
         CLC   =CL6'LENGTH',0(6)  LENGTH PARAMETER?                     CA104340
         BNE   PARS016            NO, CHECK FOR NEXT TYPE               CA104350
         MVI   TYPEFLG,C'L'       SET FOR LENGTH NAME                   CA104360
         B     PARS020            GO CHECK FOR VALUE                    CA104370
PARS016  DS    0H                                                       CA104380
         CLC   =CL7'TRANSID',0(6) TRANSID PARAMETER?                    CA104390
         BNE   PARS017            NO, INVALID PARAMETER TYPE            CA104400
         MVI   TYPEFLG,C'T'       SET FOR TRANSID NAME                  CA104410
         B     PARS020            GO CHECK FOR VALUE                    CA104420
PARS017  DS    0H                                                       CA104430
         MVC   RETCODE,=F'-5'     SET INVALID PARAMETER TYPE            CA104440
         B     PARS099                                                  CA104450
PARS020  DS    0H                                                       CA104460
         LA    4,1(0,4)           POINT TO NEXT CHARACTER               CA104470
         LR    6,4                SAVE STARTING ADDRESS                 CA104480
         SR    7,7                CLEAR PARM PIECE LENGTH               CA104490
         BCT   5,PARS021          DECREASE COUNT BY 1                   CA104500
         B     PARS070            NONE, GO TO NEXT ARGUMENT             CA104510
PARS021  DS    0H                                                       CA104520
         CLI   0(4),C')'          LOOK FOR CLOSE PAREN                  CA104530
         BE    PARS030            FOUND CLOSE PAREN                     CA104540
         LA    4,1(0,4)           POINT TO NEXT CHARACTER               CA104550
         LA    7,1(0,7)           INCREASE COUNT OF CHARS               CA104560
         BCT   5,PARS021          LOOK AT NEXT CHARACTER                CA104570
         B     PARS070            NO ')' FOUND, GO TO NEXT ARGUMENT     CA104580
PARS030  DS    0H                                                       CA104590
         CLI   TYPEFLG,C'P'       PROGRAM PARAMETER?                    CA104600
         BNE   PARS031            NO, GO TO THE NEXT TYPE               CA104610
         LTR   7,7                ANY CHARACTERS FOUND?                 CA104620
         BZ    PARS050            NO, INVALID PROGRAM NAME              CA104630
         C     7,=F'8'            IS NAME > 8 CHARACTERS                CA104640
         BH    PARS050            YES, INVALID PROGRAM NAME             CA104650
         BCTR  7,0                DECREASE FOR EX INSTRUCTION           CA104660
         EX    7,MVCPGMN          SAVE PROGRAM NAME                     CA104670
         B     PARS040            GO TO NEXT ARGUMENT                   CA104680
MVCPGMN  MVC   PGMNAME(0),0(6)    SAVE PROGRAM NAME                     CA104690
PARS031  DS    0H                                                       CA104700
         CLI   TYPEFLG,C'A'       APPLID PARAMETER?                     CA104710
         BNE   PARS032            NO, GO TO THE NEXT TYPE               CA104720
         LTR   7,7                ANY CHARACTERS FOUND?                 CA104730
         BZ    PARS051            NO, INVALID APPLID                    CA104740
         C     7,=F'8'            IS NAME > 8 CHARACTERS                CA104750
         BH    PARS051            YES, INVALID APPLID                   CA104760
         BCTR  7,0                DECREASE FOR EX INSTRUCTION           CA104770
         EX    7,MVCAPPL          SAVE APPLID NAME                      CA104780
         B     PARS040            GO TO NEXT ARGUMENT                   CA104790
MVCAPPL  MVC   APPLID(0),0(6)     SAVE APPLID NAME                      CA104800
PARS032  DS    0H                                                       CA104810
         CLI   TYPEFLG,C'U'       USERID PARAMETER?                     CA104820
         BNE   PARS033            NO, GO TO THE NEXT TYPE               CA104830
         LTR   7,7                ANY CHARACTERS FOUND?                 CA104840
         BZ    PARS056            NO, INVALID USERID                    CA104850
         C     7,=F'8'            IS NAME > 8 CHARACTERS                CA104860
         BH    PARS056            YES, INVALID USERID                   CA104870
         BCTR  7,0                DECREASE FOR EX INSTRUCTION           CA104880
         EX    7,MVCUID           SAVE USERID NAME                      CA104890
         B     PARS040            GO TO NEXT ARGUMENT                   CA104900
MVCUID   MVC   USERID(0),0(6)     SAVE USERID NAME                      CA104910
PARS033  DS    0H                                                       CA104920
         CLI   TYPEFLG,C'C'       COMMAREA PARAMETER?                   CA104930
         BNE   PARS034            NO, GO TO THE NEXT TYPE               CA104940
         LTR   7,7                ANY CHARACTERS SPECIFIED?             CA104950
         BZ    PARS055            NO, REXX NAME NOT SPECIFIED           CA104960
         ST    6,CANAMEP          SAVE PTR TO CA NAME                   CA104970
         ST    7,CANAMEL          SAVE CA NAME LENGTH                   CA104980
         B     PARS040                                                  CA104990
PARS034  DS    0H                                                       CA105000
         CLI   TYPEFLG,C'L'       LENGTH PARAMETER?                     CA105010
         BNE   PARS035            NO, GO TO THE NEXT ARGUMENT           CA105020
         LTR   7,7                ANY CHARACTERS SPECIFIED?             CA105030
         BZ    PARS053            NO, DEFAULT TO NO LENGTH ENTERED      CA105040
         C     7,=F'5'            MORE THAN 5 CHARACTERS SPECIFIED?     CA105050
         BH    PARS053            YES, INVALID LENGTH SPECIFIED         CA105060
         BCTR  7,0                                                      CA105070
         EX    7,PACKINST         PACK INTO AREA                        CA105080
         CVB   7,PACKDATA         CONVERT TO BINARY                     CA105090
         C     7,=F'32768'        IS LENGTH > 32768?                    CA105100
         BH    PARS053            YES, INVALID LENGTH SPECIFIED         CA105110
         ST    7,CA_LENGTH        SET TO IGNORE LENGTH                  CA105120
         B     PARS040            GO TO NEXT ARGUMENT                   CA105130
PACKINST PACK  PACKDATA,0(0,6)    CONVERT TO PACKED FORMAT              CA105140
PARS035  DS    0H                                                       CA105150
         CLI   TYPEFLG,C'D'       DATALENGTH PARAMETER?                 CA105160
         BNE   PARS036            NO, GO TO THE NEXT ARGUMENT           CA105170
         LTR   7,7                ANY CHARACTERS SPECIFIED?             CA105180
         BZ    PARS054            NO, INVALID DATALENGTH                CA105190
         C     7,=F'5'            MORE THAN 5 CHARACTERS SPECIFIED?     CA105200
         BH    PARS054            YES, INVALID DATALENGTH               CA105210
         BCTR  7,0                                                      CA105220
         EX    7,PACKINST         PACK INTO AREA                        CA105230
         CVB   7,PACKDATA         CONVERT TO BINARY                     CA105240
         C     7,=F'32768'        IS LENGTH > 32768?                    CA105250
         BH    PARS054            YES, INVALID DATALENGTH               CA105260
         ST    7,CA_DATALENGTH    SET TO IGNORE DATALENGTH              CA105270
         B     PARS040            GO TO NEXT ARGUMENT                   CA105280
PARS036  DS    0H                 TRANSID PARAMETER?                    CA105290
         LTR   7,7                ANY CHARACTERS FOUND?                 CA105300
         BZ    PARS052            NO, INVALID TRANSID                   CA105310
         C     7,=F'4'            IS NAME > 4 CHARACTERS                CA105320
         BH    PARS052            YES, INVALID TRANSID                  CA105330
         BCTR  7,0                DECREASE FOR EX INSTRUCTION           CA105340
         EX    7,MVCTRAN          SAVE TRANSID NAME                     CA105350
         B     PARS040            GO TO NEXT ARGUMENT                   CA105360
MVCTRAN  MVC   TRANSID(0),0(6)    SAVE TRANSID NAME                     CA105370
PARS040  DS    0H                                                       CA105380
         LA    4,1(0,4)           POINT TO NEXT CHARACTER               CA105390
         BCT   5,PARS002          LOOK AT NEXT CHARACTER                CA105400
         B     PARS070            NO ')' FOUND, GO TO NEXT ARGUMENT     CA105410
PARS050  DS    0H                                                       CA105420
         MVC   RETCODE,=F'-1'     INVALID PROGRAM NAME SPECIFIED        CA105430
         B     PARS099                                                  CA105440
PARS051  DS    0H                                                       CA105450
         MVC   RETCODE,=F'-2'     INVALID APPLID SPECIFIED              CA105460
         B     PARS099                                                  CA105470
PARS052  DS    0H                                                       CA105480
         MVC   RETCODE,=F'-6'     INVALID TRANSID SPECIFIED             CA105490
         B     PARS099                                                  CA105500
PARS053  DS    0H                                                       CA105510
         MVC   RETCODE,=F'-7'     INVALID LENGTH SPECIFIED              CA105520
         B     PARS099                                                  CA105530
PARS054  DS    0H                                                       CA105540
         MVC   RETCODE,=F'-8'     INVALID DATALENGTH SPECIFIED          CA105550
         B     PARS099                                                  CA105560
PARS055  DS    0H                                                       CA105570
         MVC   RETCODE,=F'-9'     INVALID COMMAREA REXX NAME SPECIFIED  CA105580
         B     PARS099                                                  CA105590
PARS056  DS    0H                                                       CA105600
         MVC   RETCODE,=F'-10'    INVALID USERID SPECIFIED              CA105610
         B     PARS099                                                  CA105620
PARS070  DS    0H                                                       CA105630
         LA    3,8(0,3)           POINT TO NEXT ARGUMENT                CA105640
         B     PARS001            PROCESS NEXT ARGUMENT                 CA105650
PARS080  DS    0H                                                       CA105660
         CLC   PGMNAME,=CL8' '    WAS PROGRAM NAME ENTERED?             CA105670
         BNE   PARS081            YES, CONTINUE                         CA105680
         MVC   RETCODE,=F'-1'     SET RETURN CODE                       CA105690
         B     PARS099            RETURN ERROR CODE                     CA105700
PARS081  DS    0H                                                       CA105710
         CLC   APPLID,=CL8' '     WAS APPLID ENTERED?                   CA105720
         BNE   PARS082            YES, CONTINUE                         CA105730
         MVC   RETCODE,=F'-2'     SET RETURN CODE                       CA105740
         B     PARS099            RETURN ERROR CODE                     CA105750
PARS082  DS    0H                                                       CA105760
         CLC   CA_LENGTH,=F'0'    WAS LENGTH SPECIFIED?                 CA105770
         BE    PARS099            ..NO, DO NOT CHECK DATALENGTH         CA105780
         CLC   CA_DATALENGTH,=F'0'  WAS DATALENGTH SPECIFIED?           CA105790
         BNE   PARS084            ..NO, THEN SET DATALENGTH TO LENGTH   CA105800
         MVC   CA_LENGTH,CA_DATALENGTH  SET DATALENGTH = LENGTH         CA105810
         B     PARS099                                                  CA105820
PARS084  DS    0H                 LENGTH SPECIFIED                      CA105830
         CLC   CA_LENGTH,CA_DATALENGTH  MAKE SURE LENGTH ^< DATALENGTH  CA105840
         BNL   PARS099            LENGTHS SPECIFIED ARE OK              CA105850
         MVC   RETCODE,=F'-8'     SET INVALID DATALENGTH SPECIFIED      CA105860
         B     PARS099            RETURN ERROR CODE                     CA105870
PARS099  DS    0H                                                       CA105880
         CLC   RETCODE,=F'0'      PARSE ERRORS?                         CA105890
         BNE   RETURN             ..YES, RETURN ERROR CODE              CA105900
         BR    9                  RETURN TO CALLER                      CA105910
*********************************************************************** CA105920
*   GET ADDRESS TO IRXEXCOM ROUTINE                                   * CA105930
*                                                                     * CA105940
*   REGISTER USAGE:                                                   * CA105950
*      0 - ENTRY POINT OF LOADED ROUTINE                              * CA105960
*      1 - WORK REGISTER                                              * CA105970
*      9 - RETURN TO CALLER ADDRESS                                   * CA105980
*     14 - WORK REGISTER                                              * CA105990
*     15 - RETURN CODE FROM LOAD                                      * CA106000
*********************************************************************** CA106010
LOAD_IRXEXCOM  DS 0H                                                    CA106020
         LOAD  EP=IRXEXCOM        LOAD REXX VARIABLE ACCESS ROUTINE     CA106030
         LTR   15,15              WAS LOAD SUCCESSFUL?                  CA106040
         BZ    LOAD1              YES                                   CA106050
         MVC   RETCODE,=F'-4'     SET LOAD ERROR RETURN CODE            CA106060
         B     RETURN             RETURN ERROR MESSAGE                  CA106070
LOAD1    DS    0H                                                       CA106080
         LR    15,0               LOAD ENTRY POINT ADDRESS              CA106090
         ST    15,IRXEXCOM_ADDR   SAVE FOR OTHER USES                   CA106100
         BR    9                  RETURN TO CALLER                      CA106110
*********************************************************************** CA106120
*   CALL IRXEXCOM TO RETRIEVE REXX VARIABLE CONTENTS                  * CA106130
*                                                                     * CA106140
*   REGISTER USAGE:                                                   * CA106150
*      1 - WORK REGISTER                                              * CA106160
*      4 - POINTER TO SHVBLOCK                                        * CA106170
*      9 - RETURN TO CALLER ADDRESS                                   * CA106180
*     14 - WORK REGISTER                                              * CA106190
*     15 - BRANCH REGISTER AND RETURN CODE                            * CA106200
*********************************************************************** CA106210
GETCA    DS    0H                                                       CA106220
         LA    4,IRXSHVB0         POINT TO FIRST SHVBLOCK               CA106230
         USING SHVBLOCK,4         ESTABLISH ADDRESSABILITY              CA106240
         SR    14,14              CLEAR REGISTER                        CA106250
         ST    14,SHVNEXT         SET NO MORE SHVBLOCKS                 CA106260
         ST    14,SHVUSER         INITIALIZE USER FIELD                 CA106270
         MVI   SHVCODE,SHVFETCH   SET CODE TO FETCH VARIABLE            CA106280
         L     14,CANAMEP         LOAD VAR NAME PTR                     CA106290
         ST    14,SHVNAMA         SAVE IN SHVBLOCK                      CA106300
         L     14,CANAMEL                    LOAD VAR NAME LENGTH       CA106310
         ST    14,SHVNAML         SET VARIABLE NAME LENGTH              CA106320
         L     14,BUFADDR         LOAD COMMAREA BUFFER PTR              CA106330
         ST    14,SHVVALA         SAVE IN SHVBLOCK                      CA106340
         L     14,=F'32768'       LOAD COMMAREA BUFFER LENGTH           CA106350
         ST    14,SHVBUFL         SAVE IN SHVBLOCK                      CA106360
         LA    1,IRXNAME          POINT TO IRXNAME PARAMETER            CA106370
         ST    1,EXP1ADDR         SAVE IN 1ST ADDRESS                   CA106380
         SR    1,1                CLEAR REG 1                           CA106390
         ST    1,EXP2ADDR         SAVE IN 2ND ADDRESS                   CA106400
         ST    1,EXP3ADDR         SAVE IN 3RD ADDRESS                   CA106410
         LA    1,IRXSHVB0         POINT TO SHVBLOCK                     CA106420
         ST    1,EXP4ADDR         SAVE IN 4TH ADDRESS                   CA106430
         LA    1,ENVBADDR         POINT TO CALLERS ENVIRONMENT BLOCK    CA106440
         ST    1,EXP5ADDR         SAVE IN 5TH ADDRESS                   CA106450
         LA    1,RETCODE          POINT TO RETURN CODE AREA             CA106460
         O     1,=X'80000000'     SET LAST ADDRESS                      CA106470
         ST    1,EXP6ADDR         SAVE IN 5TH ADDRESS                   CA106480
         LA    1,EXCOMPRM         POINT TO PARAMETER ADDRESSES          CA106490
         L     15,IRXEXCOM_ADDR   LOAD ADDRESS OF IRXEXCOM              CA106500
         BASSM 14,15              GO GET THE REXX VARIABLE              CA106510
         LTR   15,15              SUCCESSFUL?                           CA106520
         BNZ   GETCA90            ..NO, SET ERROR AND RETURN            CA106530
         CLI   SHVRET,SHVCLEAN    INDIVIDUAL RETURN CODE GOOD?          CA106540
         BNE   GETCA90            ..NO, SET ERROR AND RETURN            CA106550
         CLI   SHVRET,SHVNEWV     VARIABLE DID NOT EXIST?               CA106560
         BE    GETCA89            ..YES, SET TO ZERO LENGTH             CA106570
         L     14,SHVVALL         LOAD RETRIEVED LENGTH                 CA106580
         ST    14,CA_RETVD_LEN    SAVE FOR LATER USE                    CA106590
GETCA89  DS    0H                                                       CA106600
         BR    9                  RETURN TO CALLER                      CA106610
GETCA90  DS    0H                                                       CA106620
         L     14,=F'-3'          SET INVALID COMMAREA RETURN CODE      CA106630
         ST    14,RETCODE         SAVE IT IN RETCODE                    CA106640
         B     RETURN             RETURN ERROR CODE                     CA106650
         DROP  4                                                        CA106660
*********************************************************************** CA106670
*   FORMAT SHARED VARIABLE BLOCK FOR STORE REQUEST                    * CA106680
*                                                                     * CA106690
*   REGISTER USAGE:                                                   * CA106700
*      0 - LENGTH OF AREA TO FREEMAIN                                 * CA106710
*      1 - ADDRESS OF AREA TO FREEMAIN                                * CA106720
*      9 - RETURN TO CALLER ADDRESS AND BRANCH AND LINK REGISTER      * CA106730
*     14 - WORK REGISTER                                              * CA106740
*     15 - WORK REGISTER                                              * CA106750
*********************************************************************** CA106760
FORMAT_SHVB    DS 0H                                                    CA106770
         L     14,SHVBKID         LOAD SHVB ID TO FORMAT                CA106780
         SLL   14,5               MULTIPLY BY 32 (LENGTH OF BLOCK)      CA106790
         LA    4,IRXSHVB0         POINT TO FIRST BLOCK                  CA106800
         AR    4,14               ADD DISPLACEMENT                      CA106810
         LTR   14,14              ANY DISPLACEMENT?                     CA106820
         BZ    FORMAT1            ..NO SKIP CHAIN POINTER SET           CA106830
         LR    15,4               LOAD SHVBLOCK ADDRESS                 CA106840
         S     15,=F'32'          POINT TO PRECEEDING BLOCK             CA106850
         USING SHVBLOCK,15        ESTABLISH ADDRESSABILITY              CA106860
         ST    4,SHVNEXT          SET CHAIN POINTER IN PREV BLOCK       CA106870
         DROP  15                                                       CA106880
FORMAT1  DS    0H                                                       CA106890
         USING SHVBLOCK,4         ESTABLISH ADDRESSABILITY              CA106900
         SR    14,14              ZERO REGISTER                         CA106910
         ST    14,SHVNEXT         SET NO MORE SHVBLOCKS                 CA106920
         ST    14,SHVUSER         INITIALIZE USER FIELD                 CA106930
         MVI   SHVCODE,SHVSTORE   SET CODE TO STORE VARIABLE            CA106940
         L     14,NAMEPTR         LOAD VARIABLE NAME PTR                CA106950
         ST    14,SHVNAMA         SAVE IN SHVBLOCK                      CA106960
         L     14,NAMELEN         LOAD VARIABLE NAME LEN                CA106970
         ST    14,SHVNAML         SAVE IN SHVBLOCK                      CA106980
         L     14,VALUPTR         LOAD VALUE PTR                        CA106990
         ST    14,SHVVALA         SAVE IN SHVBLOCK                      CA107000
         L     14,VALULEN         LOAD VALUE LENGTH                     CA107010
         ST    14,SHVVALL         SAVE IN SHVBLOCK                      CA107020
         DROP  4                                                        CA107030
         BR    9                  RETURN TO CALLER                      CA107040
*********************************************************************** CA107050
*   CALL IRXEXCOM TO STORE A REXX VARIABLE CONTENTS                   * CA107060
*                                                                     * CA107070
*   REGISTER USAGE:                                                   * CA107080
*      1 - WORK REGISTER                                              * CA107090
*      9 - RETURN TO CALLER ADDRESS                                   * CA107100
*     14 - WORK REGISTER                                              * CA107110
*     15 - BRANCH REGISTER AND RETURN CODE                            * CA107120
*********************************************************************** CA107130
WRITE_REXX  DS 0H                                                       CA107140
         LA    1,IRXNAME          POINT TO IRXNAME PARAMETER            CA107150
         ST    1,EXP1ADDR         SAVE IN 1ST ADDRESS                   CA107160
         SR    1,1                CLEAR REG 1                           CA107170
         ST    1,EXP2ADDR         SAVE IN 2ND ADDRESS                   CA107180
         ST    1,EXP3ADDR         SAVE IN 3RD ADDRESS                   CA107190
         LA    1,IRXSHVB0         POINT TO SHVBLOCK                     CA107200
         ST    1,EXP4ADDR         SAVE IN 4TH ADDRESS                   CA107210
         LA    1,ENVBADDR         POINT TO CALLERS ENVIRONMENT BLOCK    CA107220
         ST    1,EXP5ADDR         SAVE IN 5TH ADDRESS                   CA107230
         LA    1,RETCODE          POINT TO RETURN CODE AREA             CA107240
         O     1,=X'80000000'     SET LAST ADDRESS                      CA107250
         ST    1,EXP6ADDR         SAVE IN 5TH ADDRESS                   CA107260
         LA    1,EXCOMPRM         POINT TO PARAMETER ADDRESSES          CA107270
         L     15,IRXEXCOM_ADDR                                         CA107280
         BASSM 14,15              GO WRITE THE REXX VARIABLES           CA107290
         BR    9                  RETURN TO CALLER                      CA107300
*********************************************************************** CA107310
*   ROUTINE TO ISSUE EXCI LINK TO CICS                                * CA107320
*                                                                     * CA107330
*   REGISTER USAGE:                                                   * CA107340
*      1 - WORK REGISTER                                              * CA107350
*      3 - POINTER TO EVALBLOCK TO RETURN TO REXX                     * CA107360
*      7 - POINTER TO COMMAREA TO PASS                                * CA107370
*      9 - RETURN TO CALLER ADDRESS                                   * CA107380
*     14 - WORK REGISTER                                              * CA107390
*     15 - BRANCH REGISTER AND RETURN CODE                            * CA107400
*********************************************************************** CA107410
CALLCICS DS    0H                                                       CA107420
*********************************************************************** CA107430
*     INITIALIZE USER FOR EXCI                                        * CA107440
*********************************************************************** CA107450
         LA    1,VERSION_1        LOAD EXCI VERSION                     CA107460
         ST    1,VERSION          SAVE IT                               CA107470
         LA    1,INIT_USER        LOAD INITIALIZE_USER COMMAND          CA107480
         ST    1,CALLTYPE         SAVE IT                               CA107490
         MVC   T,=C'INIT USER'    SET INITIALIZE USER                   CA107500
         MVC   TL,=F'9'           SET LENGTH                            CA107510
         CALL  DFHXCIS,                                                XCA107520
               (VERSION,          VERSION OF EXCI                      XCA107530
               RCAREA,            RETURN CODE AREA                     XCA107540
               UTOKEN,            USER TOKEN                           XCA107550
               CALLTYPE,          COMMAND                              XCA107560
               LINKID),           LINK NAME FOR SPECIFIC CONNECTION    XCA107570
               VL,MF=(E,PARMLIST)                                       CA107580
         L     1,RCRESP           LOAD RESPONSE CODE                    CA107590
         LTR   1,1                NON ZERO RETURN?                      CA107600
         BNZR  9                  YES, RETURN                           CA107610
*********************************************************************** CA107620
*     ALLOCATE PIPE TO CICS REGION                                    * CA107630
*********************************************************************** CA107640
         LA    1,ALLOCATE_PIPE    LOAD ALLOCATE PIPE COMMAND            CA107650
         ST    1,CALLTYPE         SAVE IT                               CA107660
         MVI   ALLOCOPT,SPECIFIC_PIPE  SET FOR SPECIFIC PIPE REQUEST    CA107670
         MVC   T,=C'ALLOC PIPE'   SET ALLOCATE PIPE                     CA107680
         MVC   TL,=F'10'          SET LENGTH                            CA107690
         CALL  DFHXCIS,                                                XCA107700
               (VERSION,          VERSION OF EXCI                      XCA107710
               RCAREA,            RETURN CODE AREA                     XCA107720
               UTOKEN,            USER TOKEN                           XCA107730
               CALLTYPE,          COMMAND                              XCA107740
               PTOKEN,            PIPE TOKEN                           XCA107750
               APPLID,            APPLID                               XCA107760
               ALLOCOPT),         ALLOCATE OPTIONS                     XCA107770
               VL,MF=(E,PARMLIST)                                       CA107780
         L     1,RCRESP           LOAD RESPONSE CODE                    CA107790
         LTR   1,1                NON ZERO RETURN?                      CA107800
         BNZR  9                  YES, RETURN                           CA107810
*********************************************************************** CA107820
*     OPEN PIPE TO CICS REGION                                        * CA107830
*********************************************************************** CA107840
         LA    1,OPEN_PIPE        LOAD OPEN PIPE COMMAND                CA107850
         ST    1,CALLTYPE         SAVE IT                               CA107860
         MVC   T,=C'OPEN PIPE'    SET OPEN PIPE                         CA107870
         MVC   TL,=F'9'           SET LENGTH                            CA107880
         CALL  DFHXCIS,                                                XCA107890
               (VERSION,          VERSION OF EXCI                      XCA107900
               RCAREA,            RETURN CODE AREA                     XCA107910
               UTOKEN,            USER TOKEN                           XCA107920
               CALLTYPE,          COMMAND                              XCA107930
               PTOKEN),           PIPE TOKEN                           XCA107940
               VL,MF=(E,PARMLIST)                                       CA107950
         L     1,RCRESP           LOAD RETURN CODE                      CA107960
         LTR   1,1                NORMAL RETURN?                        CA107970
         BNZR  9                  NO, RETURN TO CALLER                  CA107980
*********************************************************************** CA107990
*     DPL REQUEST                                                     * CA108000
*********************************************************************** CA108010
         L     7,BUFADDR          LOAD POINTER TO COMMAREA              CA108020
         LA    1,DPL_REQUEST      LOAD DPL REQUEST COMMAND              CA108030
         ST    1,CALLTYPE         SAVE IT                               CA108040
         MVI   DPLOPTS,SYNCONRETURN  SET DPL OPTION SYNCONRETURN        CA108050
         L     1,CANAMEP          SEE IF THERE IS A COMMAREA            CA108060
         LTR   1,1                                                      CA108070
         BNZ   CALL1              YES, THERE IS A COMMAREA TO PASS      CA108080
         SR    7,7                CLEAR COMMAREA POINTER                CA108090
         ST    7,CA_DATALENGTH    SET DATALENGTH TO ZERO                CA108100
         ST    7,CA_LENGTH        SET LENGTH TO ZERO                    CA108110
         B     CALL3                                                    CA108120
CALL1    DS    0H                                                       CA108130
         CLC   CA_DATALENGTH,=F'0' WAS DATALENGTH SPECIFIED?            CA108140
         BNE   CALL2              USE LENGTH SPECIFIED                  CA108150
         MVC   CA_LENGTH,=F'0'    SET DATALENGTH TO COMMAREA LENGTH     CA108160
CALL2    DS    0H                                                       CA108170
         CLC   CA_LENGTH,=F'0'    CHECK TO SEE IF LENGTH SPECIFIED      CA108180
         BNE   CALL3              USE LENGTH SPECIFIED                  CA108190
         L     14,CA_RETVD_LEN    LOAD COMMAREA RETRIEVED LENGTH        CA108200
         ST    14,CA_LENGTH       SET COMMAREA LENGTH TO REXX VARIABLE  CA108210
         ST    14,CA_DATALENGTH   SET COMMAREA DATALENGTH TO REXX VAR   CA108220
CALL3    DS    0H                                                       CA108230
         LA    3,TRANSID          POINT TO THE TRANSID                  CA108240
         CLC   TRANSID,=CL4' '    IS TRANSID SPECIFIED?                 CA108250
         BNE   CALL4              YES                                   CA108260
         LA    3,0                SETUP NULL POINTER                    CA108270
CALL4    DS    0H                                                       CA108280
         MVC   T,=C'DPL CALL'     SET DPL CALL                          CA108290
         MVC   TL,=F'8'           SET LENGTH                            CA108300
         CALL  DFHXCIS,                                                XCA108310
               (VERSION,          VERSION OF EXCI                      XCA108320
               RCAREA,            RETURN CODE AREA                     XCA108330
               UTOKEN,            USER TOKEN                           XCA108340
               CALLTYPE,          COMMAND                              XCA108350
               PTOKEN,            PIPE TOKEN                           XCA108360
               PGMNAME,           CICS PROGRAM NAME                    XCA108370
               0(,7),             COMMAREA                             XCA108380
               CA_LENGTH,         COMMAREA LENGTH                      XCA108390
               CA_DATALENGTH,     COMMAREA DATALENGTH TO PASS          XCA108400
               0(,3),             TRANSACTION ID OF MIRROR             XCA108410
               0,                 UNIT OF WORK ID                      XCA108420
               USERID,            CICS USERID                          XCA108430
               DPLRAREA,          DPL RETURN AREA                      XCA108440
               DPLOPTS),          DPL OPTIONS                          XCA108450
               VL,MF=(E,PARMLIST)                                       CA108460
         L     1,RCRESP           LOAD RESPONSE CODE                    CA108470
         LTR   1,1                NON ZERO RETURN?                      CA108480
         BNZR  9                  YES, RETURN                           CA108490
*********************************************************************** CA108500
*     CLOSE PIPE                                                      * CA108510
*********************************************************************** CA108520
         LA    1,CLOSE_PIPE       LOAD CLOSE PIPE COMMAND               CA108530
         ST    1,CALLTYPE         SAVE IT                               CA108540
         MVC   T,=C'CLOSE PIPE'   SET CLOSE PIPE                        CA108550
         MVC   TL,=F'10'          SET LENGTH                            CA108560
         CALL  DFHXCIS,                                                XCA108570
               (VERSION,          VERSION OF EXCI                      XCA108580
               RCAREA,            RETURN CODE AREA                     XCA108590
               UTOKEN,            USER TOKEN                           XCA108600
               CALLTYPE,          COMMAND                              XCA108610
               PTOKEN),           PIPE TOKEN                           XCA108620
               VL,MF=(E,PARMLIST)                                       CA108630
         L     1,RCRESP           LOAD RESPONSE CODE                    CA108640
         LTR   1,1                NON ZERO RETURN?                      CA108650
         BNZR  9                  YES, RETURN                           CA108660
*********************************************************************** CA108670
*     DEALLOCATE PIPE                                                 * CA108680
*********************************************************************** CA108690
         LA    1,DEALLOCATE_PIPE  LOAD DEALLOCATE PIPE COMMAND          CA108700
         ST    1,CALLTYPE         SAVE IT                               CA108710
         MVC   T,=C'DEALLOC PIPE' SET DEALLOCATE PIPE                   CA108720
         MVC   TL,=F'12'          SET LENGTH                            CA108730
         CALL  DFHXCIS,                                                XCA108740
               (VERSION,          VERSION OF EXCI                      XCA108750
               RCAREA,            RETURN CODE AREA                     XCA108760
               UTOKEN,            USER TOKEN                           XCA108770
               CALLTYPE,          COMMAND                              XCA108780
               PTOKEN),           PIPE TOKEN                           XCA108790
               VL,MF=(E,PARMLIST)                                       CA108800
         L     1,RCRESP           LOAD RESPONSE CODE                    CA108810
         LTR   1,1                NON ZERO RETURN?                      CA108820
         BNZR  9                  YES, RETURN                           CA108830
         MVC   T,=C'AFTER CALL'   SET BEFORE CALL                       CA108840
         MVC   TL,=F'10'          SET LENGTH                            CA108850
         XC    RETCODE,RETCODE    CLEAR RETURN CODE                     CA108860
*********************************************************************** CA108870
*        RETURN RETCODE AREAS IN EVALBLOCK                            * CA108880
*********************************************************************** CA108890
CALLEND  DS    0H                                                       CA108900
         BR    9                  RETURN TO CALLER                      CA108910
*********************************************************************** CA108920
*   CONVERT BINARY NUMBERS TO REXX NUMBERS                            * CA108930
*                                                                     * CA108940
*   1 CONTAINS NUMBER TO CONVERT ON ENTRY.                            * CA108950
*                                                                     * CA108960
*   ON EXIT VARIABLES CHARNUM CONTAINS THE NUMBER                     * CA108970
*                     CHARNUM_LEN CONTAINS THE LENGTH                 * CA108980
*                                                                     * CA108990
*   REGISTER USAGE:                                                   * CA109000
*      1 - WORK REGISTER - VALUE TO CONVERT                           * CA109010
*      7 - WORK REGISTER - POINTER TO CONVERTED NUMBER (INPUT)        * CA109020
*      8 - WORK REGISTER - POINTER TO CONVERTED NUMBER (OUTPUT)       * CA109030
*      9 - RETURN TO CALLER ADDRESS                                   * CA109040
*     14 - WORK REGISTER                                              * CA109050
*     15 - WORK REGISTER - COUNT OF RESULTING CHARACTERS              * CA109060
*********************************************************************** CA109070
CVT_NUM  DS    0H                                                       CA109080
         LTR   1,1                CHECK FOR ZERO                        CA109090
         BZ    CVT_NUM0           IT'S ZERO                             CA109100
         CVD   1,PACKDATA         CONVERT TO DECIMAL                    CA109110
         UNPK  CHARNUM,PACKDATA   UNPACK DATA                           CA109120
         OI    CHARNUM+15,X'F0'   SET TO NUMERIC CHARACTER              CA109130
         LA    7,CHARNUM          POINT TO FIRST CHAR POSITION          CA109140
         LR    8,7                POINT TO FIRST CHAR POSITION          CA109150
         SR    15,15              SET COUNT TO ZERO                     CA109160
         L     14,=F'15'          SET COUNT OF CHARACTERS               CA109170
         LTR   1,1                CHECK FOR NEGATIVE NO                 CA109180
         BNM   CVT_NUM1           ..NOT MINUS NO SIGN NEEDED            CA109190
         MVI   0(8),C'-'          MOVE IN MINUS SIGN                    CA109200
         LA    8,1(0,8)           SET TO NEXT CHARACTER POSITION        CA109210
         LA    15,1(0,15)         ADD 1 TO COUNT                        CA109220
CVT_NUM1 DS    0H                                                       CA109230
         LA    7,1(0,7)           INCREMENT POSITION TO NEXT CHAR       CA109240
         CLI   0(7),C'0'          IS THE CHARACTER ZERO?                CA109250
         BNE   CVT_NUM2           1ST NON ZERO DETECTED                 CA109260
         BCT   14,CVT_NUM1        CHECK THE NEXT CHARACTER              CA109270
         DC    H'0'               SHOULD NEVER GET HERE                 CA109280
CVT_NUM2 DS    0H                                                       CA109290
         AR    15,14              ADJUST FINAL COUNT                    CA109300
         BCTR  14,0               DECREASE COUNT                        CA109310
         EX    14,MOVE_CVT        MOVE CHARACTERS                       CA109320
         ST    15,CHARNUM_LEN     SAVE FINAL COUNT                      CA109330
         BR    9                  RETURN TO CALLER                      CA109340
MOVE_CVT MVC   0(0,8),0(7)        SHIFT CHARACTERS TO LEFT              CA109350
CVT_NUM0 DS    0H                                                       CA109360
         MVI   CHARNUM,C'0'       SET TO ZERO                           CA109370
         MVC   CHARNUM_LEN,=F'1'  SET LENGTH TO 1                       CA109380
         BR    9                  RETURN TO CALLER                      CA109390
*********************************************************************** CA109400
*        CONSTANTS AND LITERALS                                       * CA109410
*********************************************************************** CA109420
IRXNAME  DC    CL8'IRXEXCOM'      'IRXEXCOM' CHARACTER STREAM           CA109430
EXCI_RESP      DC CL9'EXCI_RESP'     'EXCI_RESP' REXX VARIABLE NAME     CA109440
EXCI_RESP_L    DC F'9'               'EXCI_RESP' REXX VARIABLE NAME LEN CA109450
EXCI_RESP2     DC CL10'EXCI_RESP2'   'EXCI_RESP2' REXX VAR NAME         CA109460
EXCI_RESP2_L   DC F'10'              'EXCI_RESP2' REXX VAR NAME LEN     CA109470
EXCI_ABCODE    DC CL11'EXCI_ABCODE'  'EXCI_ABCODE' REXX VAR NAME        CA109480
EXCI_ABCODE_L  DC F'11'              'EXCI_ABCODE' REXX VAR NAME LEN    CA109490
EXCI_MESSAGE   DC CL12'EXCI_MESSAGE' 'EXCI_MESSAGE' REXX VAR NAME       CA109500
EXCI_MESSAGE_L DC F'12'              'EXCI_MESSAGE' REXX VAR NAME LEN   CA109510
DPL_RESP       DC CL8'DPL_RESP'    'DPL_RESP' REXX VARIABLE NAME        CA109520
DPL_RESP_L     DC F'8'             'DPL_RESP' REXX VARIABLE NAME LEN    CA109530
DPL_RESP2      DC CL9'DPL_RESP2'   'DPL_RESP2' REXX VAR NAME            CA109540
DPL_RESP2_L    DC F'9'             'DPL_RESP2' REXX VAR NAME LEN        CA109550
DPL_ABCODE     DC CL10'DPL_ABCODE' 'DPL_ABCODE' REXX VAR NAME           CA109560
DPL_ABCODE_L   DC F'10'            'DPL_ABCODE' REXX VAR NAME LEN       CA109570
TRACE_LOC      DC CL9'TRACE_LOC'   'TRACE_LOC' REXX VARIABLE NAME       CA109580
TRACE_LOC_L    DC F'9'            'TRACE_LOC' REXX VARIABLE NAME LEN    CA109590
LINKID         DC CL8'CA1DCONN'   LINKID FOR INIT CALL                  CA109600
         LTORG                                                          CA109610
*********************************************************************** CA109620
*        DYNAMIC STORAGE AREAS FOR PROGRAM                            * CA109630
*********************************************************************** CA109640
*                                                                       CA109650
DFHEISTG DSECT                                                          CA109660
*********************************************************************** CA109670
*        RETURN CODE AREA FOR EXCI RETURN CODES                       * CA109680
*********************************************************************** CA109690
RCAREA   DS    0CL(EXCI_EXEC_RETURN_CODE__LEN)                          CA109700
RCRESP   DS    F                  RETCODE RESP AREA                     CA109710
RCRESP2  DS    F                  RETCODE RESP2 AREA                    CA109720
RCABCODE DS    F                  RETCODE ABCODE AREA                   CA109730
RCMSGLEN DS    F                  RETCODE MSGLEN AREA                   CA109740
RCMSGPTR DS    F                  RETCODE MSGPTR AREA                   CA109750
RCRESPC  DS    CL16' '            CHARACTER FIELD FOR RESP              CA109760
RCRESP2C DS    CL16' '            CHARACTER FIELD FOR RESP2             CA109770
*********************************************************************** CA109780
*        AREAS FOR CALL INTERFACE COMMANDS                            * CA109790
*********************************************************************** CA109800
VERSION  DS    F                  VERSION OF EXCI                       CA109810
UTOKEN   DS    F                  USER TOKEN                            CA109820
PTOKEN   DS    F                  PIPE TOKEN                            CA109830
CALLTYPE DS    F                  FUNCTION OF THE COMMAND               CA109840
DPLRAREA DS    0XL12              DPL RETURN AREA                       CA109850
DPLRESP  DS    F                  DPL RESPONSE CODE                     CA109860
DPLRESP2 DS    F                  DPL RESPONSE2 CODE                    CA109870
DPLABEND DS    CL4                DPL ABEND CODE                        CA109880
DPLRESPC DS    CL16               CHARACTER FIELD FOR DPLRESP           CA109890
DPLRESP2C DS   CL16               CHARACTER FIELD FOR DPLRESP2          CA109900
PARMLIST DS    14F                PARAMETER LIST                        CA109910
DPLOPTS  DS    X                  DPL OPTIONS                           CA109920
ALLOCOPT DS    X                  ALLOCATE OPTIONS                      CA109930
*********************************************************************** CA109940
*        PARAMETER LIST FOR IRXEXCOM CALL                             * CA109950
*********************************************************************** CA109960
EXCOMPRM DS    0F                 IRXEXCOM PARAMETER LIST               CA109970
EXP1ADDR DS    F                  ADDRESS OF 1ST PARAMETER              CA109980
EXP2ADDR DS    F                  ADDRESS OF 2ND PARAMETER              CA109990
EXP3ADDR DS    F                  ADDRESS OF 3RD PARAMETER              CA110000
EXP4ADDR DS    F                  ADDRESS OF 4TH PARAMETER              CA110010
EXP5ADDR DS    F                  ADDRESS OF 5TH PARAMETER              CA110020
EXP6ADDR DS    F                  ADDRESS OF 6TH PARAMETER              CA110030
*********************************************************************** CA110040
*        PARSE PARAMETER AREAS FROM REXX CALL                         * CA110050
*********************************************************************** CA110060
RETCODE                   DS F    RETURN CODE FROM IRXEXCOM             CA110070
CANAMEP  DS    F                  CA VARIABLE NAME PTR                  CA110080
CANAMEL  DS    F                  CA VARIABLE NAME LENGTH               CA110090
IRXEXCOM_ADDR             DS F    IRXEXCOM ADDRESS FOR EXECUTION        CA110100
PGMNAME                   DS CL8  CICS PROGRAM NAME                     CA110110
TRANSID                   DS CL4  CICS TRANSACTION ID                   CA110120
APPLID                    DS CL8  CICS APPLID                           CA110130
USERID                    DS CL8  CICS USERID                           CA110140
UOWID                     DS CL27 UNIT OF WORK ID                       CA110150
CA_LENGTH                 DS F    COMMAREA LENGTH                       CA110160
CA_DATALENGTH             DS F    DATA LENGTH OF COMMAREA               CA110170
*********************************************************************** CA110180
*        MISCELLANEOUS AREAS                                          * CA110190
*********************************************************************** CA110200
ENVBADDR DS    F                  REXX ENVIRONMENT BLOCK ADDRESS        CA110210
BUFADDR  DS    F                  COMMAREA BUFFER ADDRESS               CA110220
DATALEN  DS    F                  LENGTHS IN BINARY FORMAT              CA110230
NAMEPTR  DS    F                  PTR TO REXX VAR NAME                  CA110240
NAMELEN  DS    F                  LENGTH OF REXX VAR NAME               CA110250
VALUPTR  DS    F                  PTR TO VARIABLE VALUE                 CA110260
VALULEN  DS    F                  LENGTH OF VARIABLE VALUE              CA110270
SHVBKID  DS    F                  SHVBLOCK INDICATOR                    CA110280
REXXOUT_RET DS F                  SAVEAREA FOR RETURN ADDRESS           CA110290
CHARNUM_LEN DS F                  CHARACTER COUNT FOR RETURNED VALUE    CA110300
CA_RETVD_LEN DS F                 COMMAREA RETRIEVED LENGTH             CA110310
IRXP2    DS    CL4                IRXEXCOM PARAMETER 2                  CA110320
IRXSHVB0 DS    CL32               IRXEXCOM PARAMETER 0 (SHVBLOCK)       CA110330
IRXSHVB1 DS    CL32               IRXEXCOM PARAMETER 1 (SHVBLOCK)       CA110340
IRXSHVB2 DS    CL32               IRXEXCOM PARAMETER 2 (SHVBLOCK)       CA110350
IRXSHVB3 DS    CL32               IRXEXCOM PARAMETER 3 (SHVBLOCK)       CA110360
IRXSHVB4 DS    CL32               IRXEXCOM PARAMETER 4 (SHVBLOCK)       CA110370
IRXSHVB5 DS    CL32               IRXEXCOM PARAMETER 5 (SHVBLOCK)       CA110380
IRXSHVB6 DS    CL32               IRXEXCOM PARAMETER 6 (SHVBLOCK)       CA110390
IRXSHVB7 DS    CL32               IRXEXCOM PARAMETER 7 (SHVBLOCK)       CA110400
CHARNUM  DS    CL16               CHARACTER DATA FOR RETURNED VALUE     CA110410
PACKDATA DS    D                  PACKED DECIMAL AREA FOR LENGTHS       CA110420
TL       DS    F                  LENGTH OF TRACE_LOC VALUE             CA110430
T        DS    CL20               TRACE_LOC VALUE                       CA110440
TYPEFLG  DS    C                  TYPE OF PARAMETER FLAG                CA110450
         END   CA1DCALL                                                 CA110460
