*ASM XOPTS(EXCI)                                                        CA100010
*********************************************************************** CA100020
*   PROGRAM: CA1DLINK - INTERFACE BETWEEN REXX EXEC AND CICS          * CA100030
*   LANGUAGE: ASSEMBLER                                               * CA100040
*                                                                     * CA100050
*   FUNCTION: PROVIDE CICS EXCI DYNAMIC PROGRAM LINK                  * CA100060
*                                                                     * CA100070
*   FORMAT OF CALL FROM REXX:                                         * CA100080
*     CA1DLINK(PROGRAM(PROGRAM_NAME) ,                                * CA100090
*              APPLID(VTAM_APPLID) ,                                  * CA100100
*              COMMAREA(CA_NAME) ,          OPTIONAL                  * CA100110
*              LENGTH(CA_LENGTH) ,          OPTIONAL                  * CA100120
*              DATALENGTH(CA_DATALENGTH) ,  OPTIONAL                  * CA100130
*              TRANSID(TRANS_ID)            OPTIONAL                  * CA100140
*                                                                     * CA100150
*   WHERE:                                                            * CA100160
*     PROGRAM_NAME IS THE CICS PROGRAM NAME TO BE CALLED              * CA100170
*     VTAM_APPLID IS THE VTAM APPLID FOR THE CICS REGION TO ROUTE     * CA100180
*       THE REQUEST TO.                                               * CA100190
*     CA_NAME IS THE REXX VARIABLE NAME WHICH IS THE COMMAREA DATA    * CA100200
*       IF NOT SPECIFIED, THEN ASSUME NO COMMAREA TO BE RETURNED.     * CA100210
*     CA_LENGTH IS THE LENGTH OF THE COMMAREA                         * CA100220
*       IF NOT SPECIFIED THEN ASSUME LENGTH OF REXX COMMAREA VARIABLE.* CA100230
*     CA_DATALENGTH IS THE LENGTH OF THE COMMAREA TO BE PASSED        * CA100240
*       STARTING WITH THE FIRST CHARACTER OF THE COMMAREA.            * CA100250
*     TRANS_ID IS THE TRANSACTION NAME FOR THE MIRROR TRANSACTION.    * CA100260
*       CSMI USED IF NOT ENTERED                                      * CA100270
*                                                                     * CA100280
*   REXX VARIABLES USED:                                              * CA100290
*     CA_NAME WILL BE REPLACED UPON SUCCESSFUL EXECUTION OF THE       * CA100300
*       CALLED PROGRAM WITH THE RETURNED COMMAREA.                    * CA100310
*     EXCI_RESP IS THE RESP CODE RETURNED BY THE EXCI.  FORMAT IS     * CA100320
*       DECIMAL NUMERIC.                                              * CA100330
*     EXCI_RESP2 IS THE RESP2 CODE RETURNED BY THE EXCI.  FORMAT IS   * CA100340
*       DECIMAL NUMERIC.                                              * CA100350
*     EXCI_ABCODE IS THE ABCODE RETURNED BY THE EXCI.  FORMAT IS      * CA100360
*       4 BYTES CHARACTER.                                            * CA100370
*     EXCI_MESSAGE IS ANY MESSAGE RETURNED BY THE EXCI.  THIS AREA    * CA100380
*       WILL CONTAIN A NULL STRING IF CICS DOES NOT RETURN A MESSAGE. * CA100390
*                                                                     * CA100400
*   RETURN CODES: (RESULT)                                            * CA100410
*     -1 = PROGRAM NAME NOT SPECIFIED OR INVALID                      * CA100420
*     -2 = VTAM APPLID NOT SPECIFIED OR INVALID                       * CA100430
*     -3 = COMMAREA NOT AVAILABLE                                     * CA100440
*     -4 = LOAD OF IRXEXCOM FAILED                                    * CA100450
*     -5 = INVALID PARAMETER TYPE ENTERED                             * CA100460
*     -6 = TRANSID INVALID                                            * CA100470
*     -7 = LENGTH INVALID                                             * CA100480
*     -8 = DATALENGTH INVALID                                         * CA100490
*     -9 = INVALID REXX NAME FOR COMMAREA SPECIFIED                   * CA100500
*      0 = PROGRAM RECEIVED NORMAL RETURN FROM EXEC CICS EXCI         * CA100510
*          EXCI_... VARIABLES CONTAIN RESULTING RETURN CODES          * CA100520
*          (SEE EXTERNAL CICS INTERFACE FOR RETURNED VARIABLES.)      * CA100530
*                                                                     * CA100540
*   REGISTER USAGE:                                                   * CA100550
*      1 - WORK REGISTER                                              * CA100560
*      2 - POINTER TO PARAMETERS PASSED BY REXX                       * CA100570
*      9 - BRANCH AND LINK REGISTER TO SUBROUTINES                    * CA100580
*     12 - BASE REGISTER                                              * CA100590
*     13 - DFHEISTG REGISTER (DYNAMIC STORAGE AREA)                   * CA100600
*     14 - WORK REGISTER                                              * CA100610
*     15 - WORK REGISTER                                              * CA100620
*********************************************************************** CA100630
*********************************************************************** CA100640
*   INCLUDE THE STANDARD COPYBOOKS FOR EXCI DSECTS                    * CA100650
*********************************************************************** CA100660
          COPY DFHXCRCD                                                 CA100670
          EJECT                                                         CA100680
          COPY DFHXCPLD                                                 CA100690
          EJECT                                                         CA100700
*********************************************************************** CA100710
*   INCLUDE THE REXX COPYBOOKS FOR DSECTS                             * CA100720
*********************************************************************** CA100730
          IRXSHVB                 SHARED VARIABLE BLOCK                 CA100740
          EJECT                                                         CA100750
          IRXEVALB                EVALUATION BLOCK                      CA100760
          EJECT                                                         CA100770
          IRXEFPL                 EXTERNAL FUNCTION PARAMETER LIST      CA100780
          EJECT                                                         CA100790
          IRXARGTB                PARSED PARAMETER ARGUMENT LIST        CA100800
          EJECT                                                         CA100810
*********************************************************************** CA100820
*        CA1DLINK PROGRAM START                                       * CA100830
*********************************************************************** CA100840
CA1DLINK DFHEIENT CODEREG=(12),DATAREG=(13)                             CA100850
CA1DLINK AMODE 31                                                       CA100860
CA1DLINK RMODE ANY                                                      CA100870
         LR    2,1                SET R2 TO PARM REG                    CA100880
         USING EFPL,2             SET ADDRESSABILITY                    CA100890
         L     1,DFHEISTG+4       LOAD OLD SAVEAREA ADDRESS             CA100900
         L     1,20(0,1)          LOAD REXX ENVBLOCK ADDRESS            CA100910
         ST    1,ENVBADDR         SAVE REXX ENVBLOCK ADDRESS            CA100920
*********************************************************************** CA100930
*        MAIN CODE FOR PROGRAM                                        * CA100940
*********************************************************************** CA100950
         SR    1,1                CLEAR WORK REGISTER                   CA100960
         ST    1,RETCODE          SET ZERO RETURN CODE                  CA100970
         ST    1,BUFADDR          SET ZERO ADDRESS                      CA100980
         BAL   9,LOAD_IRXEXCOM    GO GET IRXEXCOM ROUTINE ADDRESS       CA100990
         BAL   9,PARSEPRM         GO PARSE PARAMETERS                   CA101000
         BAL   9,GETBUFF          GET BUFFER FOR COMMAREA               CA101010
         L     1,CANAMEP          LOAD POINTER TO COMMAREA NAME         CA101020
         LTR   1,1                ANY POINTER THERE?                    CA101030
         BZ    CALL               ..NO, SKIP GETTING COMMAREA           CA101040
         BAL   9,GETCA            GET COMMAREA FROM REXX                CA101050
CALL     DS    0H                                                       CA101060
         BAL   9,CALLCICS         ISSUE EXCI LINK TO CICS               CA101070
         BAL   9,REXXOUT          GO FORMAT AND WRITE REXX VARIABLES    CA101080
*********************************************************************** CA101090
*        RETURN PARSE ERROR TO CALLER                                 * CA101100
*********************************************************************** CA101110
RETURN   DS    0H                                                       CA101120
         L     1,RETCODE          LOAD RESPONSE CODE                    CA101130
         BAL   9,CVT_NUM          CONVERT TO REXX VALUE                 CA101140
         L     3,EFPLEVAL                                               CA101150
         L     3,0(0,3)           LOAD EVALBLOCK ADDRESS                CA101160
         USING EVALBLOCK,3        ESTABLISH ADDRESSABILITY              CA101170
         L     14,EVALBLOCK_EVSIZE                LOAD BLOCK SIZE       CA101180
         SLL   14,3                               MULTIPLY BY 8         CA101190
         S     14,=F'16'                          REDUCE BY 16          CA101200
         C     14,=F'16'                          IS AREA > 16?         CA101210
         BL    RETURN9                      ..NO, DO NOT SET EVALBLOCK  CA101220
         MVC   EVALBLOCK_EVDATA(16),CHARNUM       MOVE IN VALUE         CA101230
         MVC   EVALBLOCK_EVLEN,CHARNUM_LEN        SET LENGTH OF VALUE   CA101240
         DROP  3                                                        CA101250
RETURN9  DS    0H                                                       CA101260
*********************************************************************** CA101270
*        RETURN TO CALLER                                             * CA101280
*********************************************************************** CA101290
         BAL   9,FREEBUFF         FREE AREA FOR COMMAREA                CA101300
         SR    15,15              SET RETURN CODE TO ZERO               CA101310
         DFHEIRET RCREG=(15)                                            CA101320
         EJECT                                                          CA101330
*********************************************************************** CA101340
*   GETMAIN AREA TO BE USED FOR COMMAREA                              * CA101350
*                                                                     * CA101360
*   REGISTER USAGE:                                                   * CA101370
*      0 - LENGTH OF AREA TO GETMAIN                                  * CA101380
*      1 - RETURNED ADDRESS OF AREA                                   * CA101390
*      9 - RETURN TO CALLER ADDRESS                                   * CA101400
*     14 - WORK REGISTER                                              * CA101410
*     15 - WORK REGISTER AND RETURN CODE                              * CA101420
*********************************************************************** CA101430
GETBUFF  DS    0H                                                       CA101440
         L     0,=F'32768'        GET AREA FOR MAX COMMAREA             CA101450
         GETMAIN R,LV=(0)         GET THE AREA                          CA101460
         ST    1,BUFADDR          SAVE BUFFER AREA ADDRESS              CA101470
         BR    9                  RETURN TO CALLER                      CA101480
         SPACE 2                                                        CA101490
*********************************************************************** CA101500
*   FREEMAIN AREA USED FOR COMMAREA                                   * CA101510
*                                                                     * CA101520
*   REGISTER USAGE:                                                   * CA101530
*      0 - LENGTH OF AREA TO FREEMAIN                                 * CA101540
*      1 - ADDRESS OF AREA TO FREEMAIN                                * CA101550
*      9 - RETURN TO CALLER ADDRESS                                   * CA101560
*     14 - WORK REGISTER                                              * CA101570
*     15 - WORK REGISTER AND RETURN CODE                              * CA101580
*********************************************************************** CA101590
FREEBUFF DS    0H                                                       CA101600
         L     0,=F'32768'        GET AREA FOR MAX COMMAREA             CA101610
         L     1,BUFADDR          LOAD ADDRESS OF BUFFER                CA101620
         LTR   1,1                IS THERE AN ADDRESS TO FREE?          CA101630
         BZR   9                  ..NO, RETURN TO CALLER                CA101640
         FREEMAIN R,LV=(0),A=(1)  FREE THE AREA                         CA101650
         BR    9                  RETURN TO CALLER                      CA101660
         EJECT                                                          CA101670
*********************************************************************** CA101680
*   FORMAT SHVBLOCKS AND WRITE REXX VARIABLES                         * CA101690
*                                                                     * CA101700
*   REGISTER USAGE:                                                   * CA101710
*      0 - LENGTH OF AREA TO FREEMAIN                                 * CA101720
*      1 - ADDRESS OF AREA TO FREEMAIN                                * CA101730
*      9 - RETURN TO CALLER ADDRESS AND BRANCH AND LINK REGISTER      * CA101740
*     14 - WORK REGISTER                                              * CA101750
*     15 - WORK REGISTER                                              * CA101760
*********************************************************************** CA101770
REXXOUT  DS    0H                                                       CA101780
*                                                                       CA101790
*        FORMAT BLOCK 0 (EXCI_RESP)                                     CA101800
*                                                                       CA101810
         ST    9,REXXOUT_RET      SAVE RETURN ADDRESS                   CA101820
         LA    14,EXCI_RESP       LOAD NAME POINTER                     CA101830
         ST    14,NAMEPTR         SAVE                                  CA101840
         L     14,EXCI_RESP_L     LOAD NAME LENGTH                      CA101850
         ST    14,NAMELEN         SAVE                                  CA101860
         L     1,RCRESP           LOAD VALUE POINTER                    CA101870
         BAL   9,CVT_NUM          CONVERT TO REXX VALUE                 CA101880
         MVC   RCRESPC,CHARNUM    SAVE CHARACTER VALUE OF RCRESP        CA101890
         LA    14,RCRESPC         POINT TO CHARACTER AREA               CA101900
         ST    14,VALUPTR         SAVE                                  CA101910
         L     14,CHARNUM_LEN     LOAD VALUE LENGTH                     CA101920
         ST    14,VALULEN         SAVE                                  CA101930
         SR    14,14              SET 0 BLOCK INDICATOR                 CA101940
         ST    14,SHVBKID         SAVE                                  CA101950
         BAL   9,FORMAT_SHVB      GO FORMAT BLOCK                       CA101960
*                                                                       CA101970
*        FORMAT BLOCK 1 (EXCI_RESP2)                                    CA101980
*                                                                       CA101990
         LA    14,EXCI_RESP2      LOAD NAME POINTER                     CA102000
         ST    14,NAMEPTR         SAVE                                  CA102010
         L     14,EXCI_RESP2_L    LOAD NAME LENGTH                      CA102020
         ST    14,NAMELEN         SAVE                                  CA102030
         L     1,RCRESP2          LOAD VALUE POINTER                    CA102040
         BAL   9,CVT_NUM          CONVERT TO REXX VALUE                 CA102050
         MVC   RCRESP2C,CHARNUM   SAVE CHARACTER VALUE OF RCRESP        CA102060
         LA    14,RCRESP2C        POINT TO RETURNED VALUE               CA102070
         ST    14,VALUPTR         SAVE                                  CA102080
         L     14,CHARNUM_LEN     LOAD VALUE LENGTH                     CA102090
         ST    14,VALULEN         SAVE                                  CA102100
         L     14,SHVBKID         LOAD CURRENT BLOCK INDICATOR          CA102110
         LA    14,1(14)           INCREMENT BY 1                        CA102120
         ST    14,SHVBKID         SAVE                                  CA102130
         BAL   9,FORMAT_SHVB      GO FORMAT BLOCK                       CA102140
*                                                                       CA102150
*        FORMAT BLOCK 2 (EXCI_ABCODE)                                   CA102160
*                                                                       CA102170
         LA    14,EXCI_ABCODE     LOAD NAME POINTER                     CA102180
         ST    14,NAMEPTR         SAVE                                  CA102190
         L     14,EXCI_ABCODE_L   LOAD NAME LENGTH                      CA102200
         ST    14,NAMELEN         SAVE                                  CA102210
         LA    14,RCABCODE        LOAD VALUE POINTER                    CA102220
         ST    14,VALUPTR         SAVE                                  CA102230
         L     14,=F'4'           LOAD VALUE LENGTH                     CA102240
         ST    14,VALULEN         SAVE                                  CA102250
         L     14,SHVBKID         LOAD CURRENT BLOCK INDICATOR          CA102260
         LA    14,1(14)           INCREMENT BY 1                        CA102270
         ST    14,SHVBKID         SAVE                                  CA102280
         BAL   9,FORMAT_SHVB      GO FORMAT BLOCK                       CA102290
*                                                                       CA102300
*        FORMAT BLOCK 3 (EXCI_MESSAGE)                                  CA102310
*                                                                       CA102320
         LA    14,EXCI_MESSAGE    LOAD NAME POINTER                     CA102330
         ST    14,NAMEPTR         SAVE                                  CA102340
         L     14,EXCI_MESSAGE_L  LOAD NAME LENGTH                      CA102350
         ST    14,NAMELEN         SAVE                                  CA102360
         LA    14,RCMSGPTR        LOAD VALUE POINTER                    CA102370
         ST    14,VALUPTR         SAVE                                  CA102380
         L     14,RCMSGLEN        LOAD MESSAGE LENGTH                   CA102390
         ST    14,VALULEN         SAVE VALUE LENGTH                     CA102400
         L     14,SHVBKID         LOAD CURRENT BLOCK INDICATOR          CA102410
         LA    14,1(14)           INCREMENT BY 1                        CA102420
         ST    14,SHVBKID         SAVE                                  CA102430
         BAL   9,FORMAT_SHVB      GO FORMAT BLOCK                       CA102440
*                                                                       CA102450
*        FORMAT BLOCK 4 (COMMAREA)                                      CA102460
*                                                                       CA102470
         L     15,CA_LENGTH       LOAD COMMAREA LENGTH                  CA102480
         LTR   15,15              ANY AREA SPECIFIED?                   CA102490
         BZ    REXXOUT9           ..NO, FINISHED FORMATTING             CA102500
         L     14,CANAMEP         LOAD PTR TO NAME                      CA102510
         ST    14,NAMEPTR         SAVE                                  CA102520
         L     14,CANAMEL         LOAD LENGTH OF NAME                   CA102530
         ST    14,NAMELEN         SAVE                                  CA102540
         L     14,BUFADDR         LOAD PTR TO COMMAREA                  CA102550
         ST    14,VALUPTR         SAVE                                  CA102560
         L     14,CA_LENGTH       LOAD VALUE LENGTH                     CA102570
         ST    14,VALULEN         SAVE                                  CA102580
         L     14,SHVBKID         LOAD CURRENT BLOCK INDICATOR          CA102590
         LA    14,1(14)           INCREMENT BY 1                        CA102600
         ST    14,SHVBKID         SAVE                                  CA102610
         ST    14,SHVBKID         SAVE                                  CA102620
         BAL   9,FORMAT_SHVB      GO FORMAT BLOCK                       CA102630
REXXOUT9 DS    0H                                                       CA102640
         BAL   9,WRITE_REXX       GO WRITE REXX VARIABLES               CA102650
         L     9,REXXOUT_RET      LOAD RETURN ADDRESS                   CA102660
         BR    9                  RETURN TO CALLER                      CA102670
         EJECT                                                          CA102680
*********************************************************************** CA102690
*   PARSE PARAMETERS PASSED FROM REXX                                 * CA102700
*                                                                     * CA102710
*   FORMAT FOR CALL FROM REXX EXEC IS:                                * CA102720
*                                                                     * CA102730
*   X = CA1DLINK( PROGRAM(PGMNAME),                                   * CA102740
*                 APPLID(APPL_ID),                                    * CA102750
*                 COMMAREA(CA_VARNAME),           OPTIONAL            * CA102760
*                 TRANSID(TRAN_ID),               OPTIONAL            * CA102770
*                 DATALENGTH(DATA_LENGTH_VALUE),  OPTIONAL            * CA102780
*               )                                                     * CA102790
*                                                                     * CA102800
*   WHERE:                                                            * CA102810
*                                                                     * CA102820
*     PGMNAME IS THE NAME OF THE CICS PROGRAM TO LINK TO              * CA102830
*     CA_VARNAME IS THE NAME OF THE REXX VARIABLE WHICH IS THE        * CA102840
*       THE COMMAREA                                                  * CA102850
*     TRAN_ID IS THE TRANSACTION ID FOR THE MIRROR                    * CA102860
*     APPL_ID IS THE VTAM APPLID OF THE CICS REGION TO ROUTE TO       * CA102870
*     DATA_LENGTH_VALUE IS THE DATALENGTH TO PASS FOR THE COMMAREA    * CA102880
*                                                                     * CA102890
*   REGISTER USAGE:                                                   * CA102900
*      0 - WORK REGISTER                                              * CA102910
*      1 - WORK REGISTER                                              * CA102920
*      3 - ARGUMENT TABLE POINTER                                     * CA102930
*      4 - ARGUMENT POINTER                                           * CA102940
*      5 - ARGUMENT LENGTH                                            * CA102950
*      6 - WORK REGISTER                                              * CA102960
*      7 - WORK REGISTER                                              * CA102970
*      9 - RETURN REGISTER                                            * CA102980
*     15 - WORK REGISTER                                              * CA102990
*********************************************************************** CA103000
PARSEPRM DS    0H                                                       CA103010
         L     3,EFPLARG          POINT TO ARGS TABLE                   CA103020
         USING ARGTABLE_ENTRY,3   ESTABLISH ADDRESSABILITY              CA103030
*        INITIALIZE FIELDS                                              CA103040
         MVC   PGMNAME,=CL8' '    CLEAR AREA WITH BLANKS                CA103050
         MVC   APPLID,=CL8' '     CLEAR AREA WITH BLANKS                CA103060
         MVC   TRANSID,=CL4' '    CLEAR AREA WITH BLANKS                CA103070
         SR    15,15              CLEAR REG 15                          CA103080
         ST    15,RETCODE         SET GOOD RETURN CODE                  CA103090
         ST    15,CANAMEP         SAVE IN CA NAME PTR                   CA103100
         ST    15,CANAMEL         SAVE IN CA NAME LENGTH                CA103110
         ST    15,CA_LENGTH       SAVE IN LENGTH VALUE                  CA103120
         ST    15,CA_DATALENGTH   SAVE IN DATALENGTH VALUE              CA103130
         ST    15,CA_RETVD_LEN    SAVE RETRIEVED COMMAREA LENGTH        CA103140
PARS001  DS    0H                 START OF LOOP                         CA103150
         CLC   ARGTABLE_ARGSTRING_PTR(8),=XL8'FFFFFFFFFFFFFFFF'         CA103160
         BE    PARS080            ALL PARAMETERS PROCESSED              CA103170
         L     4,ARGTABLE_ARGSTRING_PTR     LOAD POINTER TO ARGUMENT    CA103180
         L     5,ARGTABLE_ARGSTRING_LENGTH  LOAD ARGUMENT LENGTH        CA103190
         LTR   5,5                TEST FOR ZERO LENGTH                  CA103200
         BZ    PARS070            ZERO LENGTH, GO TO NEXT ARGUMENT      CA103210
         LR    6,4                SAVE STARTING ADDRESS                 CA103220
         SR    7,7                CLEAR PARM PIECE LENGTH               CA103230
PARS002  DS    0H                 START OF LOOP                         CA103240
         CLI   0(4),C' '          LOOK FOR LEADING BLANKS               CA103250
         BNE   PARS003            NOT A BLANK, CONTINUE                 CA103260
         LA    4,1(0,4)           BLANK SKIP IT                         CA103270
         LR    6,4                SAVE NEW STARTING POSITION            CA103280
         BCT   5,PARS002          GO LOOK AT NEXT CHARACTER             CA103290
         B     PARS070            NO MORE CHARACTERS LEFT               CA103300
PARS003  DS    0H                                                       CA103310
         CLI   0(4),C'('          LOOK FOR OPEN PAREN                   CA103320
         BE    PARS010            FOUND OPEN PAREN                      CA103330
         LA    4,1(0,4)           POINT TO NEXT CHARACTER               CA103340
         LA    7,1(0,7)           INCREASE COUNT OF CHARS               CA103350
         BCT   5,PARS003          LOOK AT NEXT CHARACTER                CA103360
         B     PARS070            NO '(' FOUND, GO TO NEXT ARGUMENT     CA103370
PARS010  DS    0H                                                       CA103380
         CLC   =CL7'PROGRAM',0(6) PROGRAM PARAMETER?                    CA103390
         BNE   PARS011            NO, CHECK FOR NEXT TYPE               CA103400
         MVI   TYPEFLG,C'P'       SET FOR PROGRAM NAME                  CA103410
         B     PARS020            GO CHECK FOR VALUE                    CA103420
PARS011  DS    0H                                                       CA103430
         CLC   =CL8'COMMAREA',0(6) COMMAREA PARAMETER?                  CA103440
         BNE   PARS012            NO, CHECK FOR NEXT TYPE               CA103450
         MVI   TYPEFLG,C'C'       SET FOR COMMAREA NAME                 CA103460
         B     PARS020            GO CHECK FOR VALUE                    CA103470
PARS012  DS    0H                                                       CA103480
         CLC   =CL6'APPLID',0(6)  APPLID PARAMETER?                     CA103490
         BNE   PARS013            NO, CHECK FOR NEXT TYPE               CA103500
         MVI   TYPEFLG,C'A'       SET FOR APPLID NAME                   CA103510
         B     PARS020            GO CHECK FOR VALUE                    CA103520
PARS013  DS    0H                                                       CA103530
         CLC   =CL10'DATALENGTH',0(6)  DATALENGTH PARAMETER?            CA103540
         BNE   PARS014            NO, CHECK FOR NEXT TYPE               CA103550
         MVI   TYPEFLG,C'D'       SET FOR DATALENGTH NAME               CA103560
         B     PARS020            GO CHECK FOR VALUE                    CA103570
PARS014  DS    0H                                                       CA103580
         CLC   =CL6'LENGTH',0(6)  LENGTH PARAMETER?                     CA103590
         BNE   PARS015            NO, CHECK FOR NEXT TYPE               CA103600
         MVI   TYPEFLG,C'L'       SET FOR LENGTH NAME                   CA103610
         B     PARS020            GO CHECK FOR VALUE                    CA103620
PARS015  DS    0H                                                       CA103630
         CLC   =CL7'TRANSID',0(6) TRANSID PARAMETER?                    CA103640
         BNE   PARS016            NO, INVALID PARAMETER TYPE            CA103650
         MVI   TYPEFLG,C'T'       SET FOR TRANSID NAME                  CA103660
         B     PARS020            GO CHECK FOR VALUE                    CA103670
PARS016  DS    0H                                                       CA103680
         MVC   RETCODE,=F'-5'     SET INVALID PARAMETER TYPE            CA103690
         B     PARS099                                                  CA103700
PARS020  DS    0H                                                       CA103710
         LA    4,1(0,4)           POINT TO NEXT CHARACTER               CA103720
         LR    6,4                SAVE STARTING ADDRESS                 CA103730
         SR    7,7                CLEAR PARM PIECE LENGTH               CA103740
         BCT   5,PARS021          DECREASE COUNT BY 1                   CA103750
         B     PARS070            NONE, GO TO NEXT ARGUMENT             CA103760
PARS021  DS    0H                                                       CA103770
         CLI   0(4),C')'          LOOK FOR CLOSE PAREN                  CA103780
         BE    PARS030            FOUND CLOSE PAREN                     CA103790
         LA    4,1(0,4)           POINT TO NEXT CHARACTER               CA103800
         LA    7,1(0,7)           INCREASE COUNT OF CHARS               CA103810
         BCT   5,PARS021          LOOK AT NEXT CHARACTER                CA103820
         B     PARS070            NO ')' FOUND, GO TO NEXT ARGUMENT     CA103830
PARS030  DS    0H                                                       CA103840
         CLI   TYPEFLG,C'P'       PROGRAM PARAMETER?                    CA103850
         BNE   PARS031            NO, GO TO THE NEXT TYPE               CA103860
         LTR   7,7                ANY CHARACTERS FOUND?                 CA103870
         BZ    PARS050            NO, INVALID PROGRAM NAME              CA103880
         C     7,=F'8'            IS NAME > 8 CHARACTERS                CA103890
         BH    PARS050            YES, INVALID PROGRAM NAME             CA103900
         BCTR  7,0                DECREASE FOR EX INSTRUCTION           CA103910
         EX    7,MVCPGMN          SAVE PROGRAM NAME                     CA103920
         B     PARS040            GO TO NEXT ARGUMENT                   CA103930
MVCPGMN  MVC   PGMNAME(0),0(6)    SAVE PROGRAM NAME                     CA103940
PARS031  DS    0H                                                       CA103950
         CLI   TYPEFLG,C'A'       APPLID PARAMETER?                     CA103960
         BNE   PARS032            NO, GO TO THE NEXT TYPE               CA103970
         LTR   7,7                ANY CHARACTERS FOUND?                 CA103980
         BZ    PARS051            NO, INVALID APPLID                    CA103990
         C     7,=F'8'            IS NAME > 8 CHARACTERS                CA104000
         BH    PARS051            YES, INVALID APPLID                   CA104010
         BCTR  7,0                DECREASE FOR EX INSTRUCTION           CA104020
         EX    7,MVCAPPL          SAVE APPLID NAME                      CA104030
         B     PARS040            GO TO NEXT ARGUMENT                   CA104040
MVCAPPL  MVC   APPLID(0),0(6)     SAVE APPLID NAME                      CA104050
PARS032  DS    0H                                                       CA104060
         CLI   TYPEFLG,C'T'       TRANSID PARAMETER?                    CA104070
         BNE   PARS033            NO, GO TO THE NEXT TYPE               CA104080
         LTR   7,7                ANY CHARACTERS FOUND?                 CA104090
         BZ    PARS052            NO, INVALID TRANSID                   CA104100
         C     7,=F'4'            IS NAME > 4 CHARACTERS                CA104110
         BH    PARS052            YES, INVALID TRANSID                  CA104120
         BCTR  7,0                DECREASE FOR EX INSTRUCTION           CA104130
         EX    7,MVCTRAN          SAVE TRANSID NAME                     CA104140
         B     PARS040            GO TO NEXT ARGUMENT                   CA104150
MVCTRAN  MVC   TRANSID(0),0(6)    SAVE TRANSID NAME                     CA104160
PARS033  DS    0H                                                       CA104170
         CLI   TYPEFLG,C'C'       COMMAREA PARAMETER?                   CA104180
         BNE   PARS034            NO, GO TO THE NEXT TYPE               CA104190
         LTR   7,7                ANY CHARACTERS SPECIFIED?             CA104200
         BZ    PARS055            NO, REXX NAME NOT SPECIFIED           CA104210
         ST    6,CANAMEP          SAVE PTR TO CA NAME                   CA104220
         ST    7,CANAMEL          SAVE CA NAME LENGTH                   CA104230
         B     PARS040                                                  CA104240
PARS034  DS    0H                                                       CA104250
         CLI   TYPEFLG,C'L'       LENGTH PARAMETER?                     CA104260
         BNE   PARS035            NO, GO TO THE NEXT ARGUMENT           CA104270
         LTR   7,7                ANY CHARACTERS SPECIFIED?             CA104280
         BZ    PARS053            NO, DEFAULT TO NO LENGTH ENTERED      CA104290
         C     7,=F'5'            MORE THAN 5 CHARACTERS SPECIFIED?     CA104300
         BH    PARS053            YES, INVALID LENGTH SPECIFIED         CA104310
         BCTR  7,0                                                      CA104320
         EX    7,PACKINST         PACK INTO AREA                        CA104330
         CVB   7,PACKDATA         CONVERT TO BINARY                     CA104340
         C     7,=F'32768'        IS LENGTH > 32768?                    CA104350
         BH    PARS053            YES, INVALID LENGTH SPECIFIED         CA104360
         ST    7,CA_LENGTH        SET TO IGNORE LENGTH                  CA104370
         B     PARS040            GO TO NEXT ARGUMENT                   CA104380
PACKINST PACK  PACKDATA,0(0,6)    CONVERT TO PACKED FORMAT              CA104390
PARS035  DS    0H                                                       CA104400
         CLI   TYPEFLG,C'D'       DATALENGTH PARAMETER?                 CA104410
         BNE   PARS040            NO, GO TO THE NEXT ARGUMENT           CA104420
         LTR   7,7                ANY CHARACTERS SPECIFIED?             CA104430
         BZ    PARS054            NO, INVALID DATALENGTH                CA104440
         C     7,=F'5'            MORE THAN 5 CHARACTERS SPECIFIED?     CA104450
         BH    PARS054            YES, INVALID DATALENGTH               CA104460
         BCTR  7,0                                                      CA104470
         EX    7,PACKINST         PACK INTO AREA                        CA104480
         CVB   7,PACKDATA         CONVERT TO BINARY                     CA104490
         C     7,=F'32768'        IS LENGTH > 32768?                    CA104500
         BH    PARS054            YES, INVALID DATALENGTH               CA104510
         ST    7,CA_DATALENGTH    SET TO IGNORE DATALENGTH              CA104520
         B     PARS040            GO TO NEXT ARGUMENT                   CA104530
PARS040  DS    0H                                                       CA104540
         LA    4,1(0,4)           POINT TO NEXT CHARACTER               CA104550
         BCT   5,PARS002          LOOK AT NEXT CHARACTER                CA104560
         B     PARS070            NO ')' FOUND, GO TO NEXT ARGUMENT     CA104570
PARS050  DS    0H                                                       CA104580
         MVC   RETCODE,=F'-1'     INVALID PROGRAM NAME SPECIFIED        CA104590
         B     PARS099                                                  CA104600
PARS051  DS    0H                                                       CA104610
         MVC   RETCODE,=F'-2'     INVALID APPLID SPECIFIED              CA104620
         B     PARS099                                                  CA104630
PARS052  DS    0H                                                       CA104640
         MVC   RETCODE,=F'-6'     INVALID TRANSID SPECIFIED             CA104650
         B     PARS099                                                  CA104660
PARS053  DS    0H                                                       CA104670
         MVC   RETCODE,=F'-7'     INVALID LENGTH SPECIFIED              CA104680
         B     PARS099                                                  CA104690
PARS054  DS    0H                                                       CA104700
         MVC   RETCODE,=F'-8'     INVALID DATALENGTH SPECIFIED          CA104710
         B     PARS099                                                  CA104720
PARS055  DS    0H                                                       CA104730
         MVC   RETCODE,=F'-9'     INVALID COMMAREA REXX NAME SPECIFIED  CA104740
         B     PARS099                                                  CA104750
PARS070  DS    0H                                                       CA104760
         LA    3,8(0,3)           POINT TO NEXT ARGUMENT                CA104770
         B     PARS001            PROCESS NEXT ARGUMENT                 CA104780
PARS080  DS    0H                                                       CA104790
         CLC   PGMNAME,=CL8' '    WAS PROGRAM NAME ENTERED?             CA104800
         BNE   PARS081            YES, CONTINUE                         CA104810
         MVC   RETCODE,=F'-1'     SET RETURN CODE                       CA104820
         B     PARS099            RETURN ERROR CODE                     CA104830
PARS081  DS    0H                                                       CA104840
         CLC   APPLID,=CL8' '     WAS APPLID ENTERED?                   CA104850
         BNE   PARS082            YES, CONTINUE                         CA104860
         MVC   RETCODE,=F'-2'     SET RETURN CODE                       CA104870
         B     PARS099            RETURN ERROR CODE                     CA104880
PARS082  DS    0H                                                       CA104890
         CLC   CA_LENGTH,=F'0'    WAS LENGTH SPECIFIED?                 CA104900
         BE    PARS099            ..NO, DO NOT CHECK DATALENGTH         CA104910
         CLC   CA_DATALENGTH,=F'0'  WAS DATALENGTH SPECIFIED?           CA104920
         BNE   PARS084            ..NO, THEN SET DATALENGTH TO LENGTH   CA104930
         MVC   CA_LENGTH,CA_DATALENGTH  SET DATALENGTH = LENGTH         CA104940
         B     PARS099                                                  CA104950
PARS084  DS    0H                 LENGTH SPECIFIED                      CA104960
         CLC   CA_LENGTH,CA_DATALENGTH  MAKE SURE LENGTH ^< DATALENGTH  CA104970
         BNL   PARS099            LENGTHS SPECIFIED ARE OK              CA104980
         MVC   RETCODE,=F'-8'     SET INVALID DATALENGTH SPECIFIED      CA104990
         B     PARS099            RETURN ERROR CODE                     CA105000
PARS099  DS    0H                                                       CA105010
         CLC   RETCODE,=F'0'      PARSE ERRORS?                         CA105020
         BNE   RETURN             ..YES, RETURN ERROR CODE              CA105030
         BR    9                  RETURN TO CALLER                      CA105040
*********************************************************************** CA105050
*   GET ADDRESS TO IRXEXCOM ROUTINE                                   * CA105060
*                                                                     * CA105070
*   REGISTER USAGE:                                                   * CA105080
*      0 - ENTRY POINT OF LOADED ROUTINE                              * CA105090
*      1 - WORK REGISTER                                              * CA105100
*      9 - RETURN TO CALLER ADDRESS                                   * CA105110
*     14 - WORK REGISTER                                              * CA105120
*     15 - RETURN CODE FROM LOAD                                      * CA105130
*********************************************************************** CA105140
LOAD_IRXEXCOM  DS 0H                                                    CA105150
         LOAD  EP=IRXEXCOM        LOAD REXX VARIABLE ACCESS ROUTINE     CA105160
         LTR   15,15              WAS LOAD SUCCESSFUL?                  CA105170
         BZ    LOAD1              YES                                   CA105180
         MVC   RETCODE,=F'-4'     SET LOAD ERROR RETURN CODE            CA105190
         B     RETURN             RETURN ERROR MESSAGE                  CA105200
LOAD1    DS    0H                                                       CA105210
         LR    15,0               LOAD ENTRY POINT ADDRESS              CA105220
         ST    15,IRXEXCOM_ADDR   SAVE FOR OTHER USES                   CA105230
         BR    9                  RETURN TO CALLER                      CA105240
*********************************************************************** CA105250
*   CALL IRXEXCOM TO RETRIEVE REXX VARIABLE CONTENTS                  * CA105260
*                                                                     * CA105270
*   REGISTER USAGE:                                                   * CA105280
*      1 - WORK REGISTER                                              * CA105290
*      4 - POINTER TO SHVBLOCK                                        * CA105300
*      9 - RETURN TO CALLER ADDRESS                                   * CA105310
*     14 - WORK REGISTER                                              * CA105320
*     15 - BRANCH REGISTER AND RETURN CODE                            * CA105330
*********************************************************************** CA105340
GETCA    DS    0H                                                       CA105350
         LA    4,IRXSHVB0         POINT TO FIRST SHVBLOCK               CA105360
         USING SHVBLOCK,4         ESTABLISH ADDRESSABILITY              CA105370
         SR    14,14              CLEAR REGISTER                        CA105380
         ST    14,SHVNEXT         SET NO MORE SHVBLOCKS                 CA105390
         ST    14,SHVUSER         INITIALIZE USER FIELD                 CA105400
         MVI   SHVCODE,SHVFETCH   SET CODE TO FETCH VARIABLE            CA105410
         L     14,CANAMEP         LOAD VAR NAME PTR                     CA105420
         ST    14,SHVNAMA         SAVE IN SHVBLOCK                      CA105430
         L     14,CANAMEL                    LOAD VAR NAME LENGTH       CA105440
         ST    14,SHVNAML         SET VARIABLE NAME LENGTH              CA105450
         L     14,BUFADDR         LOAD COMMAREA BUFFER PTR              CA105460
         ST    14,SHVVALA         SAVE IN SHVBLOCK                      CA105470
         L     14,=F'32768'       LOAD COMMAREA BUFFER LENGTH           CA105480
         ST    14,SHVBUFL         SAVE IN SHVBLOCK                      CA105490
         LA    1,IRXNAME          POINT TO IRXNAME PARAMETER            CA105500
         ST    1,EXP1ADDR         SAVE IN 1ST ADDRESS                   CA105510
         SR    1,1                CLEAR REG 1                           CA105520
         ST    1,EXP2ADDR         SAVE IN 2ND ADDRESS                   CA105530
         ST    1,EXP3ADDR         SAVE IN 3RD ADDRESS                   CA105540
         LA    1,IRXSHVB0         POINT TO SHVBLOCK                     CA105550
         ST    1,EXP4ADDR         SAVE IN 4TH ADDRESS                   CA105560
         LA    1,ENVBADDR         POINT TO CALLERS ENVIRONMENT BLOCK    CA105570
         ST    1,EXP5ADDR         SAVE IN 5TH ADDRESS                   CA105580
         LA    1,RETCODE          POINT TO RETURN CODE AREA             CA105590
         O     1,=X'80000000'     SET LAST ADDRESS                      CA105600
         ST    1,EXP6ADDR         SAVE IN 5TH ADDRESS                   CA105610
         LA    1,EXCOMPRM         POINT TO PARAMETER ADDRESSES          CA105620
         L     15,IRXEXCOM_ADDR   LOAD ADDRESS OF IRXEXCOM              CA105630
         BASSM 14,15              GO GET THE REXX VARIABLE              CA105640
         LTR   15,15              SUCCESSFUL?                           CA105650
         BNZ   GETCA90            ..NO, SET ERROR AND RETURN            CA105660
         CLI   SHVRET,SHVCLEAN    INDIVIDUAL RETURN CODE GOOD?          CA105670
         BNE   GETCA90            ..NO, SET ERROR AND RETURN            CA105680
         CLI   SHVRET,SHVNEWV     VARIABLE DID NOT EXIST?               CA105690
         BE    GETCA89            ..YES, SET TO ZERO LENGTH             CA105700
         L     14,SHVVALL         LOAD RETRIEVED LENGTH                 CA105710
         ST    14,CA_RETVD_LEN    SAVE FOR LATER USE                    CA105720
GETCA89  DS    0H                                                       CA105730
         BR    9                  RETURN TO CALLER                      CA105740
GETCA90  DS    0H                                                       CA105750
         L     14,=F'-3'          SET INVALID COMMAREA RETURN CODE      CA105760
         ST    14,RETCODE         SAVE IT IN RETCODE                    CA105770
         B     RETURN             RETURN ERROR CODE                     CA105780
         DROP  4                                                        CA105790
*********************************************************************** CA105800
*   FORMAT SHARED VARIABLE BLOCK FOR STORE REQUEST                    * CA105810
*                                                                     * CA105820
*   REGISTER USAGE:                                                   * CA105830
*      0 - LENGTH OF AREA TO FREEMAIN                                 * CA105840
*      1 - ADDRESS OF AREA TO FREEMAIN                                * CA105850
*      9 - RETURN TO CALLER ADDRESS AND BRANCH AND LINK REGISTER      * CA105860
*     14 - WORK REGISTER                                              * CA105870
*     15 - WORK REGISTER                                              * CA105880
*********************************************************************** CA105890
FORMAT_SHVB    DS 0H                                                    CA105900
         L     14,SHVBKID         LOAD SHVB ID TO FORMAT                CA105910
         SLL   14,5               MULTIPLY BY 32 (LENGTH OF BLOCK)      CA105920
         LA    4,IRXSHVB0         POINT TO FIRST BLOCK                  CA105930
         AR    4,14               ADD DISPLACEMENT                      CA105940
         LTR   14,14              ANY DISPLACEMENT?                     CA105950
         BZ    FORMAT1            ..NO SKIP CHAIN POINTER SET           CA105960
         LR    15,4               LOAD SHVBLOCK ADDRESS                 CA105970
         S     15,=F'32'          POINT TO PRECEEDING BLOCK             CA105980
         USING SHVBLOCK,15        ESTABLISH ADDRESSABILITY              CA105990
         ST    4,SHVNEXT          SET CHAIN POINTER IN PREV BLOCK       CA106000
         DROP  15                                                       CA106010
FORMAT1  DS    0H                                                       CA106020
         USING SHVBLOCK,4         ESTABLISH ADDRESSABILITY              CA106030
         SR    14,14              ZERO REGISTER                         CA106040
         ST    14,SHVNEXT         SET NO MORE SHVBLOCKS                 CA106050
         ST    14,SHVUSER         INITIALIZE USER FIELD                 CA106060
         MVI   SHVCODE,SHVSTORE   SET CODE TO STORE VARIABLE            CA106070
         L     14,NAMEPTR         LOAD VARIABLE NAME PTR                CA106080
         ST    14,SHVNAMA         SAVE IN SHVBLOCK                      CA106090
         L     14,NAMELEN         LOAD VARIABLE NAME LEN                CA106100
         ST    14,SHVNAML         SAVE IN SHVBLOCK                      CA106110
         L     14,VALUPTR         LOAD VALUE PTR                        CA106120
         ST    14,SHVVALA         SAVE IN SHVBLOCK                      CA106130
         L     14,VALULEN         LOAD VALUE LENGTH                     CA106140
         ST    14,SHVVALL         SAVE IN SHVBLOCK                      CA106150
         DROP  4                                                        CA106160
         BR    9                  RETURN TO CALLER                      CA106170
*********************************************************************** CA106180
*   CALL IRXEXCOM TO STORE A REXX VARIABLE CONTENTS                   * CA106190
*                                                                     * CA106200
*   REGISTER USAGE:                                                   * CA106210
*      1 - WORK REGISTER                                              * CA106220
*      9 - RETURN TO CALLER ADDRESS                                   * CA106230
*     14 - WORK REGISTER                                              * CA106240
*     15 - BRANCH REGISTER AND RETURN CODE                            * CA106250
*********************************************************************** CA106260
WRITE_REXX  DS 0H                                                       CA106270
         LA    1,IRXNAME          POINT TO IRXNAME PARAMETER            CA106280
         ST    1,EXP1ADDR         SAVE IN 1ST ADDRESS                   CA106290
         SR    1,1                CLEAR REG 1                           CA106300
         ST    1,EXP2ADDR         SAVE IN 2ND ADDRESS                   CA106310
         ST    1,EXP3ADDR         SAVE IN 3RD ADDRESS                   CA106320
         LA    1,IRXSHVB0         POINT TO SHVBLOCK                     CA106330
         ST    1,EXP4ADDR         SAVE IN 4TH ADDRESS                   CA106340
         LA    1,ENVBADDR         POINT TO CALLERS ENVIRONMENT BLOCK    CA106350
         ST    1,EXP5ADDR         SAVE IN 5TH ADDRESS                   CA106360
         LA    1,RETCODE          POINT TO RETURN CODE AREA             CA106370
         O     1,=X'80000000'     SET LAST ADDRESS                      CA106380
         ST    1,EXP6ADDR         SAVE IN 5TH ADDRESS                   CA106390
         LA    1,EXCOMPRM         POINT TO PARAMETER ADDRESSES          CA106400
         L     15,IRXEXCOM_ADDR                                         CA106410
         BASSM 14,15              GO WRITE THE REXX VARIABLES           CA106420
         BR    9                  RETURN TO CALLER                      CA106430
*********************************************************************** CA106440
*   ROUTINE TO ISSUE EXCI LINK TO CICS                                * CA106450
*                                                                     * CA106460
*   REGISTER USAGE:                                                   * CA106470
*      1 - WORK REGISTER                                              * CA106480
*      3 - POINTER TO EVALBLOCK TO RETURN TO REXX                     * CA106490
*      7 - POINTER TO COMMAREA TO PASS                                * CA106500
*      9 - RETURN TO CALLER ADDRESS                                   * CA106510
*     14 - WORK REGISTER                                              * CA106520
*     15 - BRANCH REGISTER AND RETURN CODE                            * CA106530
*********************************************************************** CA106540
CALLCICS DS    0H                                                       CA106550
         L     1,CANAMEP          SEE IF THERE IS A COMMAREA            CA106560
         LTR   1,1                                                      CA106570
         BZ    CALL3              ..NO, DON'T USE COMMAREA              CA106580
         CLC   CA_LENGTH,=F'0'    CHECK TO SEE IF LENGTH SPECIFIED      CA106590
         BNE   CALL1              USE LENGTH SPECIFIED                  CA106600
         L     14,CA_RETVD_LEN    LOAD COMMAREA RETRIEVED LENGTH        CA106610
         ST    14,CA_LENGTH       SET COMMAREA LENGTH TO REXX VARIABLE  CA106620
         ST    14,CA_DATALENGTH   SET COMMAREA DATALENGTH TO REXX VAR   CA106630
CALL1    DS    0H                                                       CA106640
         CLC   TRANSID,=CL4' '    IS TRANSID SPECIFIED?                 CA106650
         BNE   CALL2              ..YES, CALL WITH TRANSID SPECIFIED    CA106660
         L     7,BUFADDR          LOAD POINTER TO COMMAREA              CA106670
         EXEC CICS LINK PROGRAM(PGMNAME)                               XCA106680
               RETCODE(RCAREA)                                         XCA106690
               COMMAREA(0(0,7))                                        XCA106700
               LENGTH(CA_LENGTH+2)                                     XCA106710
               DATALENGTH(CA_DATALENGTH+2)                             XCA106720
               APPLID(APPLID)                                          XCA106730
               SYNCONRETURN                                             CA106740
         B     CALLEND            CONTINUE                              CA106750
CALL2    DS    0H                                                       CA106760
         L     7,BUFADDR          LOAD POINTER TO COMMAREA              CA106770
         EXEC CICS LINK PROGRAM(PGMNAME)                               XCA106780
               RETCODE(RCAREA)                                         XCA106790
               COMMAREA(0(0,7))                                        XCA106800
               LENGTH(CA_LENGTH+2)                                     XCA106810
               DATALENGTH(CA_DATALENGTH+2)                             XCA106820
               APPLID(APPLID)                                          XCA106830
               TRANSID(TRANSID)                                        XCA106840
               SYNCONRETURN                                             CA106850
         B     CALLEND            CONTINUE                              CA106860
CALL3    DS    0H                                                       CA106870
         CLC   TRANSID,=CL4' '    IS TRANSID SPECIFIED?                 CA106880
         BNE   CALL4              ..YES, CALL WITH TRANSID SPECIFIED    CA106890
         EXEC CICS LINK PROGRAM(PGMNAME)                               XCA106900
               RETCODE(RCAREA)                                         XCA106910
               APPLID(APPLID)                                          XCA106920
               SYNCONRETURN                                             CA106930
         B     CALLEND            CONTINUE                              CA106940
CALL4    DS    0H                                                       CA106950
         EXEC CICS LINK PROGRAM(PGMNAME)                               XCA106960
               RETCODE(RCAREA)                                         XCA106970
               APPLID(APPLID)                                          XCA106980
               TRANSID(TRANSID)                                        XCA106990
               SYNCONRETURN                                             CA107000
*********************************************************************** CA107010
*        RETURN RETCODE AREAS IN EVALBLOCK                            * CA107020
*********************************************************************** CA107030
CALLEND  DS    0H                                                       CA107040
         BR    9                  RETURN TO CALLER                      CA107050
*********************************************************************** CA107060
*   CONVERT BINARY NUMBERS TO REXX NUMBERS                            * CA107070
*                                                                     * CA107080
*   1 CONTAINS NUMBER TO CONVERT ON ENTRY.                            * CA107090
*                                                                     * CA107100
*   ON EXIT VARIABLES CHARNUM CONTAINS THE NUMBER                     * CA107110
*                     CHARNUM_LEN CONTAINS THE LENGTH                 * CA107120
*                                                                     * CA107130
*   REGISTER USAGE:                                                   * CA107140
*      1 - WORK REGISTER - VALUE TO CONVERT                           * CA107150
*      7 - WORK REGISTER - POINTER TO CONVERTED NUMBER (INPUT)        * CA107160
*      8 - WORK REGISTER - POINTER TO CONVERTED NUMBER (OUTPUT)       * CA107170
*      9 - RETURN TO CALLER ADDRESS                                   * CA107180
*     14 - WORK REGISTER                                              * CA107190
*     15 - WORK REGISTER - COUNT OF RESULTING CHARACTERS              * CA107200
*********************************************************************** CA107210
CVT_NUM  DS    0H                                                       CA107220
         LTR   1,1                CHECK FOR ZERO                        CA107230
         BZ    CVT_NUM0           IT'S ZERO                             CA107240
         CVD   1,PACKDATA         CONVERT TO DECIMAL                    CA107250
         UNPK  CHARNUM,PACKDATA   UNPACK DATA                           CA107260
         OI    CHARNUM+15,X'F0'   SET TO NUMERIC CHARACTER              CA107270
         LA    7,CHARNUM          POINT TO FIRST CHAR POSITION          CA107280
         LR    8,7                POINT TO FIRST CHAR POSITION          CA107290
         SR    15,15              SET COUNT TO ZERO                     CA107300
         L     14,=F'15'          SET COUNT OF CHARACTERS               CA107310
         LTR   1,1                CHECK FOR NEGATIVE NO                 CA107320
         BNM   CVT_NUM1           ..NOT MINUS NO SIGN NEEDED            CA107330
         MVI   0(8),C'-'          MOVE IN MINUS SIGN                    CA107340
         LA    8,1(0,8)           SET TO NEXT CHARACTER POSITION        CA107350
         LA    15,1(0,15)         ADD 1 TO COUNT                        CA107360
CVT_NUM1 DS    0H                                                       CA107370
         LA    7,1(0,7)           INCREMENT POSITION TO NEXT CHAR       CA107380
         CLI   0(7),C'0'          IS THE CHARACTER ZERO?                CA107390
         BNE   CVT_NUM2           1ST NON ZERO DETECTED                 CA107400
         BCT   14,CVT_NUM1        CHECK THE NEXT CHARACTER              CA107410
         DC    H'0'               SHOULD NEVER GET HERE                 CA107420
CVT_NUM2 DS    0H                                                       CA107430
         AR    15,14              ADJUST FINAL COUNT                    CA107440
         BCTR  14,0               DECREASE COUNT                        CA107450
         EX    14,MOVE_CVT        MOVE CHARACTERS                       CA107460
         ST    15,CHARNUM_LEN     SAVE FINAL COUNT                      CA107470
         BR    9                  RETURN TO CALLER                      CA107480
MOVE_CVT MVC   0(0,8),0(7)        SHIFT CHARACTERS TO LEFT              CA107490
CVT_NUM0 DS    0H                                                       CA107500
         MVI   CHARNUM,C'0'       SET TO ZERO                           CA107510
         MVC   CHARNUM_LEN,=F'1'  SET LENGTH TO 1                       CA107520
         BR    9                  RETURN TO CALLER                      CA107530
*********************************************************************** CA107540
*        CONSTANTS AND LITERALS                                       * CA107550
*********************************************************************** CA107560
IRXNAME  DC    CL8'IRXEXCOM'      'IRXEXCOM' CHARACTER STREAM           CA107570
EXCI_RESP      DC CL9'EXCI_RESP'  'EXCI_RESP' REXX VARIABLE NAME        CA107580
EXCI_RESP_L    DC F'9'            'EXCI_RESP' REXX VARIABLE NAME LEN    CA107590
EXCI_RESP2     DC CL10'EXCI_RESP2' 'EXCI_RESP2' REXX VAR NAME           CA107600
EXCI_RESP2_L   DC F'10'            'EXCI_RESP2' REXX VAR NAME LEN       CA107610
EXCI_ABCODE    DC CL11'EXCI_ABCODE' 'EXCI_ABCODE' REXX VAR NAME         CA107620
EXCI_ABCODE_L  DC F'11'             'EXCI_ABCODE' REXX VAR NAME LEN     CA107630
EXCI_MESSAGE   DC CL12'EXCI_MESSAGE' 'EXCI_MESSAGE' REXX VAR NAME       CA107640
EXCI_MESSAGE_L DC F'12'              'EXCI_MESSAGE' REXX VAR NAME LEN   CA107650
EDIT_PATTERN   DC X'40202020202020202020202020202120' EDIT PATTERN      CA107660
         LTORG                                                          CA107670
*********************************************************************** CA107680
*        DYNAMIC STORAGE AREAS FOR PROGRAM                            * CA107690
*********************************************************************** CA107700
*                                                                       CA107710
DFHEISTG DSECT                                                          CA107720
*********************************************************************** CA107730
*        RETURN CODE AREA FOR EXCI RETURN CODES                       * CA107740
*********************************************************************** CA107750
RCAREA   DS    0CL(EXCI_EXEC_RETURN_CODE__LEN)                          CA107760
RCRESP   DS    F                  RETCODE RESP AREA                     CA107770
RCRESP2  DS    F                  RETCODE RESP2 AREA                    CA107780
RCABCODE DS    F                  RETCODE ABCODE AREA                   CA107790
RCMSGLEN DS    F                  RETCODE MSGLEN AREA                   CA107800
RCMSGPTR DS    F                  RETCODE MSGPTR AREA                   CA107810
*********************************************************************** CA107820
*        PARAMETER LIST FOR IRXEXCOM CALL                             * CA107830
*********************************************************************** CA107840
EXCOMPRM DS    0F                 IRXEXCOM PARAMETER LIST               CA107850
EXP1ADDR DS    F                  ADDRESS OF 1ST PARAMETER              CA107860
EXP2ADDR DS    F                  ADDRESS OF 2ND PARAMETER              CA107870
EXP3ADDR DS    F                  ADDRESS OF 3RD PARAMETER              CA107880
EXP4ADDR DS    F                  ADDRESS OF 4TH PARAMETER              CA107890
EXP5ADDR DS    F                  ADDRESS OF 5TH PARAMETER              CA107900
EXP6ADDR DS    F                  ADDRESS OF 6TH PARAMETER              CA107910
*********************************************************************** CA107920
*        PARSE PARAMETER AREAS FROM REXX CALL                         * CA107930
*********************************************************************** CA107940
RETCODE                   DS F    RETURN CODE FROM IRXEXCOM             CA107950
CANAMEP  DS    F                  CA VARIABLE NAME PTR                  CA107960
CANAMEL  DS    F                  CA VARIABLE NAME LENGTH               CA107970
IRXEXCOM_ADDR             DS F    IRXEXCOM ADDRESS FOR EXECUTION        CA107980
PGMNAME                   DS CL8  CICS PROGRAM NAME                     CA107990
TRANSID                   DS CL4  CICS TRANSACTION ID                   CA108000
APPLID                    DS CL8  CICS APPLID                           CA108010
CA_LENGTH                 DS F    COMMAREA LENGTH                       CA108020
CA_DATALENGTH             DS F    DATA LENGTH OF COMMAREA               CA108030
*********************************************************************** CA108040
*        MISCELLANEOUS AREAS                                          * CA108050
*********************************************************************** CA108060
ENVBADDR DS    F                  REXX ENVIRONMENT BLOCK ADDRESS        CA108070
BUFADDR  DS    F                  COMMAREA BUFFER ADDRESS               CA108080
DATALEN  DS    F                  LENGTHS IN BINARY FORMAT              CA108090
NAMEPTR  DS    F                  PTR TO REXX VAR NAME                  CA108100
NAMELEN  DS    F                  LENGTH OF REXX VAR NAME               CA108110
VALUPTR  DS    F                  PTR TO VARIABLE VALUE                 CA108120
VALULEN  DS    F                  LENGTH OF VARIABLE VALUE              CA108130
SHVBKID  DS    F                  SHVBLOCK INDICATOR                    CA108140
REXXOUT_RET DS F                  SAVEAREA FOR RETURN ADDRESS           CA108150
CHARNUM_LEN DS F                  CHARACTER COUNT FOR RETURNED VALUE    CA108160
CA_RETVD_LEN DS F                 COMMAREA RETRIEVED LENGTH             CA108170
IRXP2    DS    CL4                IRXEXCOM PARAMETER 2                  CA108180
IRXSHVB0 DS    CL32               IRXEXCOM PARAMETER 4 (SHVBLOCK)       CA108190
IRXSHVB1 DS    CL32               IRXEXCOM PARAMETER 4 (SHVBLOCK)       CA108200
IRXSHVB2 DS    CL32               IRXEXCOM PARAMETER 4 (SHVBLOCK)       CA108210
IRXSHVB3 DS    CL32               IRXEXCOM PARAMETER 4 (SHVBLOCK)       CA108220
IRXSHVB4 DS    CL32               IRXEXCOM PARAMETER 4 (SHVBLOCK)       CA108230
CHARNUM  DS    CL16               CHARACTER DATA FOR RETURNED VALUE     CA108240
RCRESPC  DS    CL16               RESP CHARACTER DATA                   CA108250
RCRESP2C DS    CL16               RESP2 CHARACTER DATA                  CA108260
PACKDATA DS    D                  PACKED DECIMAL AREA FOR LENGTHS       CA108270
TYPEFLG  DS    C                  TYPE OF PARAMETER FLAG                CA108280
         END   CA1DLINK                                                 CA108290
